function TmaEpgEvents( events ) {

	var self = this;

	/**
	 * TODO: convert to Getters in future
	 */

	this.timestamp =			'';
	this.content =				'epg_events';
	this.count =				0;
	this.error =				0;
	this.requested_starttime =	'';
	this.requested_endtime =	'';
	this.requested_chtype =		0;
	this.requested_chnum =		0;
	this.requested_logoid = 	'';
	this.requested_epgsource =	'none';
	this.epgsource =			'none';
	this.event =				[];
	/** @returns {TmaChannel} */
	this.channel =				null;

	/**
	 * loops through all events
	 *
	 * @param {Function}	callback	callback function will be called for each recording with parameters (nr, event)
	 */
	this.eachEvent = function( callback ) {

		for (var i = 0; i < self.event.length; i++ ) {
			if (callback.apply( self.event[i], [i, self.event[i]]) === false) {
				break;
			}
		}
	};

	/**
	 * @param {String} name
	 * @returns {TmaFavouritesGroup}
	 */
	this.getEvent = function( eventid ) {

		for (var i = 0; i < self.event.length; i++) {
			if (self.event[i].eventid == eventid) {
				return self.event[i];
			}
		}

		return false;

	};

	// apply initial data
	if (events) {

		$.extend(true, this, events);

		if (events.event && $.isArray(events.event)) {

			/**
			 * sort events by starttime_local
			 * @param {TmaEpgEvent}	event1
			 * @param {TmaEpgEvent} event2
			 */
//			events.event.sort(function(event1, event2) {
//
//				var start1 = new Date(event1.starttime_local);
//				var start2 = new Date(event2.starttime_local);
//
//				return start1.getTime() - start2.getTime();
//
//			});

			var day = 99;
			var timestamp = (new Date(events.timestamp)).getTime();
			// convert events to TmaEpgEvent
			for (var i = 0; i < events.event.length; i++) {

				var event = new TmaEpgEvent(events.event[i], events.channel);
				this.event[i] = event;

				// Calculate extra data

				var event_start = event.startDate().getTime();
				var event_end	= event.endDate().getTime();

				// first event of day?
				event.firstOfDay = (event.startDate().getDate() > day);
				day = event.startDate().getDate();

				// current event?
				event.isCurrent = (event_start < timestamp) && (event_end > timestamp);
				if ( event.isCurrent ) {
					event.currentPercent = parseInt((timestamp - event_start) * 100 / (event_end - event_start));
				}

			}

			/**
			 * sort events by starttime_local
			 * @param {TmaEpgEvent}	event1
			 * @param {TmaEpgEvent} event2
			 */
			this.event.sort(function(event1, event2) {
				return event1.startDate().getTime() - event2.startDate().getTime();
			});

		}

	}

}

function TmaEpgEvent( event, channel ) {

	var self = this;

	var startDate = null;
	var endDate = null;

	/**
	 * TODO: convert to Getters in future
	 */

	this.eventid =			0;
	this.datastatus =		0;
	this.runningstatus =	0;
	this.starttime_local =	'';
	this.endtime_local =	'';
	this.timezone =			0;
	this.duration =			0;
	this.satindex =			0;
	this.networkid =		0;
	this.tsid =				0;
	this.serviceid =		0;
	this.svctype =			0;
	this.svcnum =			0;
	this.logoid =			'';	// epgsearch only
	this.lang =				'';
	this.parental =			0;
	this.namelength =		0;
	this.descriptionlength= 0;
	this.textlength =		0;
	this.name =				'';
	this.description =		'';
	this.text =				'';
	this.citid =			0;
	//this.contentidentifier=	0;
	//this.contentidentifier_hex= '';
	//this.genre_text =		'';
	//this.genre_code =		'';
	this.genre_value =		0;
	this.au_genre =			0;
	this.sourceflag =		0;
	this.icechannel =		0;
	this.parental_text =	'';
	/** @returns {TmaChannel} */
	this.channel =			null;

	/**
	 * get the related genre object
	 *
	 * @returns {TmaGenre}
	 */
	this.genre = function() {

		var genre = tma.genres.genre(self.genre_value);

		if ( genre ) {
			return genre;
		} else {
			return new TmaGenre();
		}

	};

	/**
	 * extended event text, where newlines are replaced by BR
	 *
	 * @returns {String}
	 */
	this.htmlText = function() {
		return self.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
	};

	/**
	 * Date object of 'starttime_local'
	 *
	 * @returns {Date}
	 */
	this.startDate = function() {
		if (!startDate) {
			startDate = new Date(self.starttime_local);
		}
		return startDate;
	};

	/**
	 * Date object of 'endtime_local'
	 *
	 * @returns {Date}
	 */
	this.endDate = function() {
		if (!endDate) {
			endDate = new Date(self.endtime_local);
		}
		return endDate;
	};

	// apply initial data
	if (event) {
		$.extend(this, event);
		if (!channel) {
			// if contructor is called without a channel object
			// and the plain event object has a logoid
			if (event.logoid) {
				// then set the channel object
				channel = tma.channel(event.logoid);
			}
		}
		if (channel) {
			this.channel = channel;
		}
	}

}
