/**
 * encapsulates file object of json response
 *
 * @param {TmaFile}	file
 */
function TmaFile( file ) {

	// empty default status
	var data = {
			name		: "",
			name_hex	: "",
			timestamp	: "",
			size		: 0,
			inode		: 0,
			attr		: 0,
			info		: 0,
			infpresent	: 0,
			tappresent	: 0,
			infdata		: null,
			tap			: null,

			sortdate	: "",
	};

	// apply initial data
	if (file) {

		if (file.infpresent) {
			data.infdata = new TmaInfData();
		}

		tma.extend(data, file);

		// generate a value to sort for date
		if ( (file.attr == 208) && (file.infpresent) && (file.infdata.header.starttime)) {
			data.sortdate = file.infdata.header.starttime;
		} else {
			data.sortdate = file.timestamp ;
		}

		if ( file.tappresent) {
			data.tap = new TmaTap(file.tap);
		}

	}

	/**
	 * TODO: convert to Getters in future
	 */

	this.name		= data.name;
	this.name_hex	= data.name_hex;
	this.timestamp	= data.timestamp;
	this.size		= data.size;
	this.inode		= data.inode;
	this.attr		= data.attr;
	this.info		= data.info;
	this.infpresent	= data.infpresent;
	this.tappresent	= data.tappresent;
	this.infdata	= data.infdata;
	/** @returns {TmaTap} */
	this.tap		= data.tap;

	this.sortdate	= data.sortdate;

	/**
	 * get file name of the file (non-UTF Toppy safe)
	 *
	 * @param {Boolean]	removeExt	remove file extension? (default = false)
	 * @returns {String}
	 */
	this.getName = function(removeExt) {

		/** @returns {String} */
		var name = (tma.config.utftoppy || !this.name_hex) ?
				this.name
				: tma.decodeHexname(this.name_hex);

		if (removeExt) {
			return name.replace(/\.[^.]+$/, '');
		} else {
			return name;
		}

	};

	this.isPlayable = function() {
		return (
				this.infpresent ||
				this.name.match(/\.rec$|\.mpg$|\.mkv$|\.ts$|\.vob$|\.m2ts$|\.divx$/)
		);
	};

	this.isTextfile = function() {
		return (
				this.name.match(/\.txt$|\.log|\.prn$|\.csv$|\.json$|\.ini$|\.sh$|\.lng$/)
		);
	};

	this.isFile = function() {
		return this.attr == 208;
	};

	//this.absolutefilename =	data.absolutefilename;

}

function TmaInfData() {

	// empty default status
	this.header			= new TmaInfHeader();
	this.service		= new TmaInfService();
	this.eventinfo		= new TmaInfEvent();
	this.exteventinfo	= new TmaInfExtEvent();
	this.tpinfo			= new TmaTpInfo();
	this.bookmarks		= new TmaInfBookmarks();

}

function TmaInfHeader() {

	var self = this;

	// empty default status
	this.starttime		= '';
	this.durationmin	= 0;
	this.durationsec	= 0;
	this.svcnum			= 0;
	this.svctype		= 0;
	this.logoid			= '';
	this.cryptflag		= 0;
	this.headerflags	= 0;
	this.headerflags2	= 0;
	this.flagtimeshift	= 0;
	this.flagcopy		= 0;

	/** @returns {TmaChannel} */
	this.channel = function() {
		return tma.channel(self.logoid);
	};

}

function TmaInfService() {

	// empty default status
	this.satindex		= 0;
	this.servicetype	= 0;
	this.tpindex		= 0;
	this.tunernum		= 0;
	this.flagskip		= 0;
	this.flaglock		= 0;
	this.flagcas		= 0;
	this.flagdel		= 0;
	this.serviceid		= 0;
	this.pmtpid			= 0;
	this.pcrpid			= 0;
	this.videopid		= 0;
	this.audiopid		= 0;
	this.servicename	= '';
	this.videostreamtype= 0;
	this.audiostreamtype= 0;

}

function TmaInfEvent() {

	// empty default status
	this.duration		= 0;
	this.eventid		= 0;
	this.starttime_utc	= '';
	this.starttime_local= '';
	this.endtime_utc	= '';
	this.endtime_local	= '';
	this.runningstatus	= 0;
	this.parental		= 0;
	this.parental_text	= '';
	this.serviceid		= 0;
	this.name			= '';
	this.description	= '';

}

function TmaInfExtEvent() {

	var self = this;

	// empty default status
	this.eventid		= 0;
	this.serviceid		= 0;
	this.text			= '';

	/**
	 * extended event text, where newlines are replaced by BR
	 *
	 * @returns {String}
	 */
	this.htmlText = function() {
		return self.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
	};

}

function TmaTpInfo() {

	// empty default status
	this.satindex			= 0;
	this.flags2				= 0;
	this.polarisation		= 0;
	this.mode				= 0;
	this.modsystem			= 0;
	this.modulation			= 0;
	this.fec				= 0;
	this.pilot				= 0;
	this.channelnumber		= 0;
	this.bandwidth			= 0;
	this.lphpstream			= 0;
	this.frequency			= 0;
	this.symbolrate			= 0;
	this.tsid				= 0;
	this.clocksync			= 0;
	this.networkid			= 0;
	this.originalnetworkid	= 0;

}

function TmaInfBookmarks() {

	// empty default status
	this.startposition	= 0;
	this.count			= 0;

}

