/**
 * encapsulates timer-list object of json response
 *
 * @param {TmaTimers}	timers
 */
function TmaTimers(timers) {

	var data = {
			count		: (timers.count)		|| 0,
			timestamp	: (timers.timestamp)	|| "",
			content		: (timers.content)		|| "timers",
			timers		: [],
	};

	// apply initial data
	if (timers) {

		if (timers.timers) {
			for (var i = 0; i < timers.timers.length; i++) {
				var timer = new TmaTimer(timers.timers[i]);
				timer.id = timer.id || TmaTimers.getUniqueId();
				data.timers.push( timer );
			}
		}

	}

	/**
	 * TODO: convert to Getters in future
	 */

	this.count		= data.count;
	this.timestamp	= data.timestamp;
	this.content	= data.content;
	this.timers		= data.timers;

	/**
	 * loops through all timers
	 *
	 * @param {Function}	callback	callback function will be called for each timer with
	 * 									parameters ({Number} index, {TmaTimer} timer)
	 */
	this.eachTimer = function( callback ) {

		for (var i = 0; i < data.timers.length; i++ ) {
			if (callback.apply( data.timers[i], [i, data.timers[i]]) === false) {
				break;
			}
		}
	};

	/**
	 * find timer by unique id
	 *
	 * @param {Number}	timerid
	 * @returns {TmaTimer}	or false
	 */
	this.find = function( timerid ) {

		for (var i = 0; i < data.timers.length; i++ ) {

			if (data.timers[i].id == timerid) {
				return data.timers[i];
			}
		}

		return false;

	};

}

// create a static function to get unique timer ids
(function() {

	/**
	 * counter value for unique IDs
	 * @see getUniqueId()
	 */
	var nextId = 0;

	/**
	 * returns a unique timer ID
	 *
	 * @returns {Number}
	 */
	TmaTimers.getUniqueId = function() {
		return ++nextId;
	};

})();
