/**
 * epgsearch module
 */

/* TODO List
	DONE - Bug with block width when (event in progress) + (timer) + (recoding in progress). - This seems to be a problem when a recording-in-progress is truncated.
	DONE - Timers and 'in progress' on different lines.
	* Timers on other networks.
	* "Check my timers" feature.
	* Make genre colouring work.
	* Show new timer when creating timers from the results list.
	DONE - Delete saved searches.
	DONE - Warn if name already exists when saving a new search.
	DONE - In the load search list, change the button to a check-box "Show SmartEPG Searches".  Toggle SE searches based on this check-box state.
	DONE - Sort the saved search list.
*/


tma.registerModule( epgsearch );
epgsearch.prototype = new tmaModule();
function epgsearch() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgsearch';

	this.pageGroup = 'epg';

	var self = this;

	this.delayQuery = null;
	this.queryRetry = 0;

	//this.TapId = 1145914136;

	this.port = 8800;

	var searchPort = 8800;

	var error_code = 0;

	var queryStartTime = new Date(0);

	//var epgsource = "pvr";

	var responseObject;

	var messageBuffer = "";

	var favGroup = "";

	//TODO - Consider using "this.path()" instead of and absolute path name
	var PVRdirectory = "/mnt/hd/ProgramFiles/Settings/WebControl/public_html/modules/epgsearch";

	var indexFileName = PVRdirectory + "/searches/index.json";

	var smartepgSearch = "/mnt/hd/ProgramFiles/Settings/SmartEPG_TMS/Search.txt";

	var searchFileName = "";

	var currentSearchName = "";

	//var largeString = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";

	//for(var xx = 0; xx < 16000; xx++){largeString = largeString + "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";}

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Build the criteria window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#timestamp').hide();

			currentSearchName = "";

			//Load the timers
			tma.apiGetTimers();
			//Load the genres
			tma.apiEpgGenre();

			var $content = $('DIV#content');

			//TODO - Remove analogue channel IDs.

			//var favGroup = tma.favourites.currentgroup;
			favGroup = tma.localSetting( 'defaultfavourites', '' );

			var favDesc = self.i18.not_set + " " + self.getFavDesc(favGroup);

			var $template = $(self.templates.criteria.render({
				epgwiz: 	self.i18,
				master:  	tma.i18,
				fav_desc:	favDesc,
				channels: 	tma.getChannels(favGroup),}))
			.appendTo($content);

			self.addTextRow();

			// jk: setup weekdays
			$('LABEL.epgwiz-weekday').epgwizWeekday();

			//Setup multi channel selector and callback for when a new favourites group is selected.
			var chSelectorOptions = {
				enableFavSelect:	true,
				selectedFavGroup:	favGroup,
				onFavChange:
					function(newFavGroup){

						if(chSelectorOptions.selectedFavGroup != newFavGroup)
						{
							chSelectorOptions.selectedFavGroup = newFavGroup;
							favGroup = newFavGroup;
							//if (window.console) {console.log("Favourite group changed to '" + newFavGroup + "'.");}

							self.fieldSet_services(newFavGroup);

/* 							var chList = tma.getChannels(newFavGroup);

							newList = "";
							$.each(chList, function(ix, channel) {
								newList = newList + '<option value="' + channel.logoid + '">' + channel.servicename + '</option>';
							});

							$('#searchservice')
								.empty()
								.append(newList)
								.change(); */
						}

					}
			};

			$content.find('#channelstext').click( function(e) {
				chSelectorOptions.selectedFavGroup = favGroup; //This is required for when a new search is loaded with a favgroup other than the currently selected group
				tma.fn.getModule('multichselect').showSelector($('#searchservice'), chSelectorOptions);

			});

			//Setup the reset icons
			$content.find('#epgwiz-reset-all').click( function(e) {
				self.resetAllCriteria();
			});

			$content.find('#epgwiz-reset-channels').click( function(e) {

				self.fieldSet_services(favGroup);

				//var $selChannels = $('#searchservice option').each(function() {
				//	$(this).prop('selected', false);
				//});
				//$('#searchservice').change();	//Generate a change event so that the text is updated.
			});

			$content.find('#epgwiz-reset-start-time').click( function(e) {

				self.fieldSet_time('starttime', '');

				//$('#starttime').timepicker('setTime', "00:00");
				//$('#starttime').val("");
				//$('#starttimetextblock').text(self.i18.not_set);
			});
			$content.find('#epgwiz-reset-end-time').click( function(e) {

				self.fieldSet_time('endtime', '');

				//$('#endtime').timepicker('setTime', "00:00");
				//$('#endtime').val("");
				//$('#endtimetextblock').text(self.i18.not_set);
			});
			$content.find('#epgwiz-reset-min-len').click( function(e) {

				self.fieldSet_time('minlen', '');

				//$('#minlen').timepicker('setTime', "00:00");
				//$('#minlen').val("");
				//$('#minlentextblock').text(self.i18.not_set);
			});
			$content.find('#epgwiz-reset-max-len').click( function(e) {

				self.fieldSet_time('maxlen', '');

				//$('#maxlen').timepicker('setTime', "00:00");
				//$('#maxlen').val("");
				//$('#maxlentextblock').text(self.i18.not_set);
			});
			$content.find('#epgwiz-reset-start-date').click( function(e) {

				self.fieldSet_date('startdate', '');

				//$('#startdate').datetimepicker('setDate', null);
				//$('#startdate').val("");
				//$('#startdatetextblock').text(self.i18.not_limited);
			});
			$content.find('#epgwiz-reset-end-date').click( function(e) {

				self.fieldSet_date('enddate', '');

				//$('#enddate').datetimepicker('setDate', null);
				//$('#enddate').val("");
				//$('#enddatetextblock').text(self.i18.not_limited);
			});
			$content.find('#epgwiz-preset-alldays').click( function(e) {
				$('LABEL.epgwiz-weekday').epgwizWeekday(true, true, true, true, true, true, true);
			});
			$content.find('#epgwiz-preset-weekdays').click( function(e) {
				$('LABEL.epgwiz-weekday').epgwizWeekday(true, true, true, true, true, false, false);
			});
			$content.find('#epgwiz-preset-weekends').click( function(e) {
				$('LABEL.epgwiz-weekday').epgwizWeekday(false, false, false, false, false, true, true);
			});
			$content.find('#epgwiz-preset-nodays').click( function(e) {
				$('LABEL.epgwiz-weekday').epgwizWeekday(false, false, false, false, false, false, false);
			});
			//END Setup the reset icons

			//Setup text area help text
			$content.find('#epgwiz-texthelp').click( function(e) {
				//alert("Help button pressed");

				if ( self.i18.module.helptext ) {

					self.getModuleHelpText().done(function(info) {

						var infoWindow = new jkqWindow();
						infoWindow.width = 580;

						infoWindow.showModal();
						infoWindow.setTitle( self.i18.module.help_title );
						infoWindow.closeOnOutsideClick();

						infoWindow.container.find('DIV.jkqWindowContent')
						.append(
								'<div class="epgwiz-help">' +
								info +
								'</div>'
						);

						infoWindow.adjustPos();

					});

				}

			});

			//Display the selected channels
			$content.find('#searchservice').change( function() {

				var chCounter = $('#searchservice :selected').length;

				//var favGroup = tma.localSetting( 'defaultfavourites', '' );
				//if (window.console) console.log("Channel list changed '" + favGroup + "' (" + chCounter.toString() + " selected).");

				var $channelsText = $('#channelstextblock');

				//var favDesc = "";
				var favDesc = self.getFavDesc(favGroup);

				if(chCounter > 0)
				{
					$channelsText.html(chCounter.toString() + " " + self.i18.selected + " " + favDesc);
				}
				else
				{
					$channelsText.html(self.i18.not_set + " " + favDesc);
				}
			});

			$content.find('#epgwiz-save-search').click( function(e) {

				new jkqInputDialog({
					title: self.i18.save,
					text: self.i18.save_long,
					inputValue: currentSearchName
				}).confirm(function() {
					self.saveSearchDefinition(this.value());
				});
				//self.saveSearchDefinition();
			});

			$content.find('#epgwiz-load-search').click( function(e) {

				//var searchName = prompt("Please enter a search name", "");
				//self.loadSearchDefinition(searchName);
				//self.loadSEDefinition();

				self.loadSearchList();

			});

			$content.find('#epgwiz-do-search').click( function(e) {

				var errorFlag = 0;

				//var result = '<table>';

				//for (var i = 0; i < 5; i++) {
				//	result = result + '<tr><td>Result number ' + i.toString() + '</td></tr>';
				//}

				//result = result + '</table>';

				var crlf = "\r\n";

				//Loop through each text row to build search string
				var textRowCount = 0;
				var textRowString = "";

				//B;*text*;50
				$('#text-criteria-table tbody tr').each(function(){

					//Work out the SmartEPG code for the text fields to be searched.
					var searchLocation = 0;
					if($(this).find('#epgwiz-search-title').prop('checked')){searchLocation = searchLocation + 1;}
					if($(this).find('#epgwiz-search-desc').prop('checked')){searchLocation = searchLocation + 10;}
					if($(this).find('#epgwiz-search-ext').prop('checked')){searchLocation = searchLocation + 100;}

					var searchLocationText = "";
					switch(searchLocation) {
						case 1:	//Only search the title
							searchLocationText = "T";
							break;
						case 10: //Only search the description
							searchLocationText = "D";
							break;
						case 100: //Only search the extended description
							searchLocationText = "E";
							break;
						case 110: //Search the descriptions only
							searchLocationText = "S";		//"S" for Synopsis, this is not an SE option but an extension for EPGapprentice
							break;
						case 101: //Only search the title and extended description
							searchLocationText = "q";		//"q" for 'this makes no sense but WC and generate this combination', this is not an SE option but an extension for EPGapprentice
							break;
						case 11: //Search the title and description
							searchLocationText = "A";
							break;
						case 111: //Search everything
							searchLocationText = "B";
							break;
					}

					if(searchLocation === 0){errorFlag = 1;}

					//if($(this).find('#epgwiz-search-text').val().length === 0){errorFlag = 2;}
					var inputText = $(this).find('#epgwiz-search-text').val();
							if( inputText.length === 0){
								errorFlag = 2;
							} else {
								// is text encased by slashes?
								var reMatch = inputText.match(/^\/(.*)\/$/);
								//If not, assume that this contains wildcards and convert to regex
								if(!reMatch)
								{
									reMatch = "/" + inputText.replace("?", ".").replace("*", ".*") + "/";
									//if (window.console) console.log("regex = " + reMatch);
								}
								if ( reMatch ) {
									// test regular expression
									try {
										//new RegExp( reMatch[1] );
										new RegExp( reMatch );
									} catch (e) {
										errorFlag = 7; // invalid regular expression
									}
								}
							}

					//textRowString = textRowString + "Search:           " + searchLocationText + ";*" + $(this).find('#epgwiz-search-text').val() + "*;" + $(this).find('#epgwiz-search-points').val() + crlf;
					textRowString = textRowString + "Search:           " + searchLocationText + ";" + $(this).find('#epgwiz-search-text').val() + ";" + $(this).find('#epgwiz-search-points').val() + crlf;
					textRowCount++;

				});//END loop through text criteria

				//Assemble the selected channels
				var chList = "";
				var chCount = 0;

				var $selChannels = $('#searchservice :selected').each(function() {
					chList += "ChannelID:        " + $(this).attr('value') + crlf;
					chCount++;
				});

				if(chCount === 0) //If no channels are selected, search them all.
				{
					if(favGroup.length === 0)
					{
						chCount = 1;
						chList = "ChannelID:        " + "0000000000000000" + crlf;
					}
					else
					{
						chList = "";
						chCount = 0;

						var chListFav = tma.getChannels(favGroup);

						$.each(chListFav, function(ix, channel) {
							chList += "ChannelID:        " + channel.logoid + crlf;
							chCount++;
						});

						//var $selChannels = $('#searchservice option').each(function() {
						//	chList += "ChannelID:        " + $(this).attr('value') + crlf;
						//	chCount++;
						//});
					}
				}

				//alert(chCount + '- ' + chList);

				//Assemble the string of selected week days
				var weekdayList = "";
				var $selDays = $('#epgwiz-weekdays INPUT:checked').each(function() {
					weekdayList += $(this).attr('value');
				});

				if(weekdayList.length === 0){errorFlag = 3;}

				//Calculate the length constraints in whole minutes
				var min_len = (new Date('2000/01/01 ' + $('#minlen').val()).getHours()) * 60 + (new Date('2000/01/01 ' + $('#minlen').val()).getMinutes());
				var max_len = (new Date('2000/01/01 ' + $('#maxlen').val()).getHours()) * 60 + (new Date('2000/01/01 ' + $('#maxlen').val()).getMinutes());

				if((max_len < min_len) && (max_len !== 0)){errorFlag = 4;}

				//Start time and end time default to "0" and "6144" respectively if nothing is specified.
				//If specified, the hours are bit-shifted 8 to the left and then combined with the minutes.

				var start_time = 0;
				var end_time = 6144;

				var temp_min = (new Date('2000/01/01 ' + $('#starttime').val()).getHours()) * 60 + (new Date('2000/01/01 ' + $('#starttime').val()).getMinutes());
				if(temp_min !== 0){start_time = Math.floor(temp_min / 60) << 8 | (temp_min % 60);}

				temp_min = (new Date('2000/01/01 ' + $('#endtime').val()).getHours()) * 60 + (new Date('2000/01/01 ' + $('#endtime').val()).getMinutes());
				if(temp_min !== 0){end_time = Math.floor(temp_min / 60) << 8 | (temp_min % 60);}

				//if(end_time < start_time){errorFlag = 5;}

				//Start date and end dated are the hex representation of the MJD for the required date.
				var start_date = self.makeMJD(new Date($('#startdate').val()));
				var temp_len = $('#startdate').val().toString().length;
				if(temp_len === 0){start_date = 0;}

				var end_date = self.makeMJD(new Date($('#enddate').val()));
				temp_len = $('#enddate').val().toString().length;
				if(temp_len === 0){end_date = 0;}

				if((end_date < start_date) && (end_date !== 0)){errorFlag = 6;}

				//*************************************************************
				//** All sanity checks should be completed before this point **
				//*************************************************************
				//Create the criteria string for EPGapprentice
				var criteria = "";

				//Because EPGapprentice is designed to read the SmartEPG Searches.txt format, we need to add some fields
				//to trick EPGapprentice into thinking that it has all of the data from Searches.txt.

				//An extension to the SmartEPG format so that EPGapprentice can malloc() its internal search structures.
				           criteria = "SearchCount:      " + "1" + crlf; //This MUST be the first field
				criteria = criteria + "Name:             " + "WebControl" + crlf;
				//An extension to the SmartEPG format so that EPGapprentice can malloc() its internal regex structures.
				criteria = criteria + "TextCount:        " + textRowCount.toString() + crlf;
				//An extension to the SmartEPG format so that EPGapprentice can malloc() its internal channel list.
				criteria = criteria + "ChannelCount:     " + chCount.toString() + crlf;
				criteria = criteria + "ShowProgress:     " + "X" + crlf;	//This value does not matter, having the field present is all that is required.
				criteria = criteria + "RecordingMode:    " + "0" + crlf;	//This MUST come immediately after the count fields and the name.
				//End trickery.

				//criteria = criteria + "Search:           " + searchLocationText + ";*" + $('#epgwiz-search-text').val() + "*;" + $('#epgwiz-search-points').val() + crlf;
				criteria = criteria + textRowString;

				//List of channelIDs
				//criteria = criteria + "ChannelID:        " + "0000000000000000" + crlf;
				criteria = criteria + chList;

				//1 = Monday 7 = Sunday
				criteria = criteria + "WeekDays:         " + weekdayList + crlf;
				//Duration in minutes
				criteria = criteria + "MinLength:        " + min_len.toString() + crlf;
				//Duration in minutes
				criteria = criteria + "MaxLength:        " + max_len.toString() + crlf;

				criteria = criteria + "StartTime:        " + start_time.toString() + crlf;
				criteria = criteria + "EndTime:          " + end_time.toString() + crlf;

				criteria = criteria + "StartDate:        " + ("0000" + start_date.toString(16)).slice(-4) + crlf;
				criteria = criteria + "EndDate:          " + ("0000" + end_date.toString(16)).slice(-4) + crlf;
				criteria = criteria + "EndQuery:         1" + crlf;

				//criteria = criteria + "Genre:            " + "43" + crlf;
				//criteria = criteria + "Genre:            " + "45" + crlf;
				//criteria = criteria + "Genre:            " + "47" + crlf;

				var byteCount = criteria.length;

                if(errorFlag === 0)
				{
					self.initepgsearchTAP().done(function() {
						//if (window.console) console.log("Initiating query delay.");
						tma.apiGetTimers();	//Refresh the timers list.  This can be asynchronous because the search will take some time to run.
						this.queryRetry = 0;
						this.delayQuery = window.setTimeout(
							function() {
								//if (window.console) console.log("Executing delayed query.");

								if(error_code === 0)
								{
									self.executeQuery(criteria, searchPort);
								}
								else
								{
									//if (window.console) console.log("Can not execute delayed query, error_code = " + error_code.toString() + ".");
									self.displayTAPBusy();
								}
							},
							750		//Wait 0.75 seconds before submitting the query to allow EPGapprentice time to start.
						);
					});
				}
				else
				{
					//alert("Error code: " + errorFlag);
					alert(self.i18.validation_errors[errorFlag]);
					/*  1 - No search fields selected
						2 - No search text provided
						3 - No weekdays specified
						4 - Inconsistent lengths
						5 - Inconsistent times
						6 - Inconsistent dates */
				}

			});  //END do-search button actions

			// setup the datepickers
			var dtpOptions = {
					//controlType: dtpSpinner,
					controlType: 'slider',
					addSliderAccess: true,
					sliderAccessArgs: { touchonly: false },
					firstDay: 1,
					//timeFormat: "HH:mm",
					timeFormat: "",
					dateFormat: "yy/mm/dd",
					monthNames: [],
					monthNamesShort: [],
					dayNames: [],
					dayNamesShort: [],
					dayNamesMin: [],
					//timeText: self.i18.dtp_time_text,
					timeText: "",
					hourText: self.i18.dtp_hour_text,
					minuteText: self.i18.dtp_minute_text,
					closeText: self.i18.dtp_close_text,
					currentText: self.i18.dtp_current_text,
					timeOnlyTitle: self.i18.dtp_time_only_title,
			};

			$.each(tma.i18.months, function(i, month) {
				dtpOptions.monthNames.push(month.name);
				dtpOptions.monthNamesShort.push(month.shortname);
			});

			$.each(tma.i18.days, function(i, day) {
				dtpOptions.dayNames.push(day.name);
				dtpOptions.dayNamesShort.push(day.shortname);
				dtpOptions.dayNamesMin.push(day.shortname.slice(0, 2));
			});

			//Setup the start date field
			$("#startdate").datetimepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					if(selectedDateTime !== null && selectedDateTime !== undefined && selectedDateTime !== '')
					{
						$("#startdatetextblock").text(tma.date(self.i18.timerdate_fmt, selectedDateTime));
					}
				}
			}));

			// click on icon or text should trigger the dt-picker
			$("#startdatetext").click(function () {
				$("#startdate").datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			//Setup the end date field
			$("#enddate").datetimepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					if(selectedDateTime !== null && selectedDateTime !== undefined && selectedDateTime !== '')
					{
						$("#enddatetextblock").text(tma.date(self.i18.timerdate_fmt, selectedDateTime));
					}
				}
			}));

			// click on icon or text should trigger the dt-picker
			$("#enddatetext").click(function () {
				$("#enddate").datetimepicker("show");
			}).css( 'cursor', 'pointer' );

			//Reconfigure dtpOptions for time-only mode
			dtpOptions.timeFormat = "HH:mm";
			dtpOptions.timeText = self.i18.dtp_time_text;
			dtpOptions.dateFormat = "HH:mm";	//The date format needs to be here or a parsing error is generated
			dtpOptions.timeOnlyTitle = self.i18.dtp_time_text;

			//Setup the start time field
			$("#starttime").timepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#starttimetextblock").text(self.timeFormatter(selectedDateTime));
				}
			}));

			$("#starttimetext").click(function () {
				$("#starttime").timepicker("show");
			}).css( 'cursor', 'pointer' );


			//Setup the end time field
			$("#endtime").timepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#endtimetextblock").text(self.timeFormatter(selectedDateTime));
				}
			}));

			$("#endtimetext").click(function () {
				$("#endtime").timepicker("show");
			}).css( 'cursor', 'pointer' );

			//Reconfigure dtpOptions for duration mode
			dtpOptions.timeOnlyTitle = self.i18.dtp_time_only_title;

			//Setup the minimum length field
			$("#minlen").timepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#minlentextblock").text(self.durationFormatter(selectedDateTime));
				}
			}));

			$("#minlentext").click(function () {
				$("#minlen").timepicker("show");
			}).css( 'cursor', 'pointer' );

			//Setup the maximum length field
			$("#maxlen").timepicker($.extend({}, dtpOptions, {
				onClose: function (selectedDateTime) {
					$("#maxlentextblock").text(self.durationFormatter(selectedDateTime));
				}
			}));

			$("#maxlentext").click(function () {
				$("#maxlen").timepicker("show");
			}).css( 'cursor', 'pointer' );

		});//END init().done function

	}; //END Show event

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		if (self.delayQuery) {
			window.clearTimeout(self.delayQuery);
			self.delayQuery = null;
		}

		$('#timestamp').show();

	};

	this.makeMJD = function(inDate) {

		//Create a MJD from JavaScript Date.
		//The JavaScript epoch is based on UTC so it is important NOT to confuse this with local time.

		var tempDate = new Date(0);

		tempDate.setUTCFullYear(inDate.getFullYear());
		tempDate.setUTCMonth(inDate.getMonth());
		tempDate.setUTCDate(inDate.getDate());
		tempDate.setUTCHours(0);
		tempDate.setUTCMinutes(0);
		tempDate.setUTCSeconds(0);

		return 40587 + (tempDate.valueOf() / 86400000);

	};

	this.MJDtoJSON = function(mjd){
		//From ETSI EN 300 468 V1.14.1 (2014-05)
		//To find Y, M, D from MJD

		//Y' = int [ (MJD - 15 078,2) / 365,25 ]
		var y1 = Math.floor((mjd - 15078.2) / 365.25);
		//M' = int { [ MJD - 14 956,1 - int (Y' × 365,25) ] / 30,6001 }
		var m1 = Math.floor((mjd - 14956.1 - (Math.floor(y1 * 365.25))) / 30.6001);
		//D = MJD - 14 956 - int (Y' × 365,25) - int (M' × 30,6001)
		var day = mjd - 14956 - (Math.floor(y1 * 365.25)) - (Math.floor(m1 * 30.6001));
		//If M' = 14 or M' = 15, then K = 1; else K = 0
		var k = 0;
		if((m1 == 14) || (m1 == 15)){k = 1;}
		//Y = Y' + K
		//if (y1 >= 100){y1 = y1 + 1900;}
		y1 = y1 + 1900;
		var year = y1 + k;
		//M = M' - 1 - K × 12
		var month = m1 - 1 - k * 12;
		//To find WD from MJD
		//WD = [ (MJD + 2) mod 7 ] + 1
		//var weekDay =  ((mjd + 2) % 7 ) + 1;

		var tDate = new Date(0);

		tDate.setFullYear(year);
		tDate.setMonth(month - 1);
		tDate.setDate(day);

		return tDate;

	};


	this.addTextRow = function(defaultValues) {

		//Add a new text criteria row to the text criteria table.
		var $table = $('TABLE#text-criteria-table');
		var $tbody = $table.find('TBODY');

		var $newRow = $(self.templates.textline.render({
			epgwiz: 	self.i18,
			master:  	tma.i18,}));

		if(defaultValues)
		{
			$newRow.find('#epgwiz-search-text').val(defaultValues.text);
			$newRow.find('#epgwiz-search-title').attr('checked', defaultValues.title);
			$newRow.find('#epgwiz-search-desc').attr('checked', defaultValues.desc);
			$newRow.find('#epgwiz-search-ext').attr('checked', defaultValues.ext);
			$newRow.find('#epgwiz-search-points').val(defaultValues.points);
		}

		$newRow.find('BUTTON.epgwiz-add-text').click( function(e) {self.addTextRow();});
		$newRow.find('BUTTON.epgwiz-delete-text').click( function(e) {self.deleteTextRow($(this).parents('TR'));});
		$newRow.appendTo($tbody);

	};//END addTextRow()

	this.deleteTextRow = function(thisRow) {

		//Delete the selected text criteria row to the text criteria table.
		//If there is only 1 row, do not allow deletion.  Maybe reset values?
		var tabRows = $('#text-criteria-table >tbody >tr').length;

		if(tabRows > 1)
		{
			thisRow.remove();
		}

	};//END deleteTextRow()

	this.timeFormatter = function(selectedTime) {

		var tempTime = new Date('2000/01/01 ' + selectedTime);

		if(tempTime.getHours() === 0 && tempTime.getMinutes() === 0)
		{
			return self.i18.not_set;
		}
		else
		{
			return tma.date(self.i18.time_fmt, tempTime);
		}

	};//END timeFormatter()

	this.durationFormatter = function(selectedTime) {

		var dur_min = ((new Date('2000/01/01 ' + selectedTime).getHours()) * 60) + (new Date('2000/01/01 ' + selectedTime).getMinutes());

		if(dur_min === 0)
		{
			return self.i18.not_set;
		}
		else
		{
			return tma.formatDuration(dur_min);
		}

	};//END durationFormatter()

	this.presentResults = function(results) {

		//var foundItems = results.count;

		//if (window.console) console.log(results.count.toString() + " results originally found");

		//var cull = 0;

		var max_epg_date = new Date();

		//Cull EPG events in the past from the object array
		$.each(results.event, function(i, evt) {

			results.event[i].ignore = false;

			//Tag events in the past for non-display
			if(new Date(evt.endtime_local) < queryStartTime)
			{
				//if (window.console) console.log("Culling (" + i.toString() + ") '" + evt.name + "' end time = " + evt.endtime_local + ", query time = " + queryStartTime);
				//cull++;
				results.event[i].ignore = true;  //I tried to delete the object here, but I has some problems so I settled on a flag instead.
				results.count--;
			}//END tag events in the past
			else
			{
				//Record the largest date returned from the search
				var temp_start_date = new Date(evt.starttime_local);
				var temp_end_date = new Date(evt.endtime_local);

				if (temp_start_date > max_epg_date){max_epg_date = temp_start_date;}
				if (temp_end_date > max_epg_date){max_epg_date = temp_end_date;}
			}
		}); //END loop through returned events.

		var foundItems = results.count;

		//if (window.console) console.log(cull.toString() + " results culled");
		//if (window.console) console.log(results.count.toString() + " results found");

		//var srt = 0;

		if (results.count > 0)
		{
			//Sort the results into start-time order
			results.event.sort(function(a, b)
			{
				//srt++;
				//if (window.console) console.log("Testing " + a.starttime_local + " vs " + b.starttime_local);
				if (a.starttime_local < b.starttime_local) return -1;
				if (a.starttime_local > b.starttime_local) return 1;
				return 0;
			});

			//if (window.console) console.log("Sort called " + srt.toString() + " times.");

			$('DIV.epgwiz DIV.epgwiz-results').html("");

			var hdr = self.i18.results_found_plural;
			if(results.count == 1)
			{hdr = self.i18.results_found_single;}

			var $result = $('DIV#epgwiz-results');

			var $resultsTable = $(self.templates.results.render({
				epgwiz: 	self.i18,
				hdr:		hdr,
				counter:	results.count,
				master:  	tma.i18,}));

			//var $tbody = $resultsTable.find('#epgwiz-results-table').children('tbody');
			var $tbody = $resultsTable.find('#epgwiz-results-body');

			var evtTmpl =	'<tr class="epgwiz-event" eventid="{eventid}" timestamp="{timestamp}">' +
								'<td class="epgwiz-clm-time">' +
									'<div class="epgwiz-clm-timebox">' +
										'<span class="channellogo channellogo-small logo-small-{logoid} epgwiz-haslogo-{haslogo}">{servicename}</span>' +
										'<span class="epgwiz-evt-date">{date}</span>' +
										'<span class="epgwiz-evt-time">{start} - {end}</span>' +
										'<span class="epgwiz-evt-duration">{dur_fmt}</span>' +
									'</div>' +
								'</td>' +
								'<td class="epgwiz-clm-name" has-descr="{hasdescr}">' +
									'{genre}' +
									'<span class="epgwiz-evt-title">{name}</span>' +
									'<span class="epgwiz-evt-descr">{description}</span>' +
									'<span class="epgwiz-evt-text">{text}</span>';
			var evtTmp2 =		'</td>' +
							'</tr>';

				var min_start_time = new Date(0);
				var max_end_time = new Date(0);

			$.each(results.event, function(i, module) {

				if(module.ignore){return true;}

				var event_start_date = new Date(this.starttime_local);
				var event_end_date	 = new Date(this.endtime_local);

				min_start_time = new Date (event_start_date);
				max_end_time = new Date (event_end_date);

				//if (window.console) console.log("Event '" + this.name + "' start time = " + this.starttime_local + "' end time = " + this.endtime_local + "'");

				//Only display events in the present or future
				//if(event_end_date < queryStartTime)
				//{
				//	//if (window.console) console.log("Ignoring '" + this.name + "' end time = " + this.endtime_local + ", query time = " + queryStartTime);
				//	foundItems--;
				//	return true;
				//}//END only display events in the present or future

				//Set the channel details for this event.
				//this.channel = tma.channel(this.logoid); // jk: channel is set by object converion
				//var svcName = this.channel.servicename; // jk: never used
				//if (window.console) console.log("Service name '" + svcName + " logoid " + this.logoid);

				//If this a event current, show the progress bar.
				//var currBarTmpl = "";
				//if ((queryStartTime <= event_end_date) && (queryStartTime >= event_start_date))
				//{
				//	var percent = parseInt((queryStartTime - event_start_date) * 100 / (event_end_date - event_start_date));

				//	currBarTmpl =		'<span class="epgwiz-evt-curr">' +
				//							'<span class="epgwiz-evt-curr-part" style="width: '+ percent.toString() +'%;" />' +
				//						'</span>';
				//}

				//var currentEvt = evtTmpl + currBarTmpl + evtTmp2;

				var tempEvent = "";
				tempEvent = this;

				// jk: genre is a method of a TmaEpgEvent
				//tempEvent.genre = new TmaGenre({
				//		genre_value:	this.genre_value,
				//		genre_code:		this.genre_code,
				//		genre_text:		this.genre_text
				//		});

				//if (window.console) console.log(tempEvent.genre);
				//if (window.console) console.log(getGenreAndParentalString(tempEvent, 1));

				//Find any overlapping timers
				var extended = 1;
				var timerBar = "";
				tempEvent.timerCount = 0;
				tempEvent.otherTimerCount = 0;
				tempEvent.timers = []; //new Array();
				tempEvent.otherTimers = []; //new Array();
				tempEvent.timers_start = []; //new Array();
				tempEvent.otherTimers_start = []; //new Array();
				$.each(tma.timers.timers, function(i, timerObj) {

					//tempTimer = this;

					tempTimer = []; //new Array();

					tempTimer = jQuery.extend(true, {}, timerObj);

					//if(this.logoid == tempEvent.logoid) //Is this the same channel?
					//{
						var timer_start = new Date(tempTimer.starttime);
						var timer_end = new Date(tempTimer.endtime);

						extended = 1;
						var extendFlag = 0;

						//if (window.console) console.log("  Timer '" + tempTimer.filename + "' start time = " + tempTimer.starttime + "' end time = " + tempTimer.endtime + "' " + extended);

						while(extended)
						{
							extended = 0;

	/*				EPG Event				EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
					  Timer A			TTTTTTTTTT
					  Timer B											TTTTTTTTT
					  Timer C						TTTTTTTTTTTTTTT
					  Timer D		TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT  */

							//if( (timer_end   >= event_start_date && timer_end <= event_end_date) ||		//A - Partial overlap at beginning of event
							//	(timer_start <= event_end_date   && timer_end >= event_end_date) ||		//B - Partial overlap at end of event
							//	(timer_start >= event_start_date && timer_end <= event_end_date) ||		//C - Event partially covered by timer
							//	(timer_start <= event_start_date && timer_end >= event_end_date) )		//D - Timer contains the whole event

							//The original logic was getting some odd matches on events where the end date matched the next start date.
							//This is the logic from TMSTimer miniEPG view that worked well there and seems to solve the problem here.

 							if(

								//Leading Event = EventStart < TimerStart & (EventEnd < TimerEnd & EventEnd > TimerStart)
								(event_start_date < timer_start && (event_end_date < timer_end && event_end_date > timer_start))  ||

								//Trailing Event = EventStart < TimerEnd & EventEnd > TimerEnd
								(event_start_date < timer_end && event_end_date > timer_end) ||

								//Partial Event = EventStart < TimerStart & EventEnd > TimerEnd
								(event_start_date < timer_start && event_end_date > timer_end) ||

								//Full Event = EventStart >= TimerEvent & EventEnd <= TimerEnd.
								(event_start_date >= timer_start && event_end_date <= timer_end) )

							{
 								//if (window.console) console.log("Timer '" + this.filename + "' matches event '" + tempEvent.name + "' " + extendFlag + " timer_start " + timer_start);

								//alert("Here");
								//if(timer_start < min_start_time){min_start_time = timer_start; console.log("set min_start " + min_start_time);}
								//if(timer_end > max_end_time){max_end_time = timer_end; console.log("set max_end");}

								//Note: The 'new' syntax is required to stop min_start_time being incremented when timer_start is incremented.
								if(timer_start < min_start_time){min_start_time = new Date(timer_start); /*console.log("set min_start " + min_start_time);*/}
								if(timer_end > max_end_time){max_end_time = new Date(timer_end); /*console.log("set max_end");*/}

								tempTimer.extended = extendFlag;

								if(this.logoid == tempEvent.logoid)
								{
									tempEvent.timers.push(tempTimer);
									//tempEvent.timers.push(jQuery.extend(true, {}, tempTimer));
									tempEvent.timers_start[tempEvent.timerCount] = new Date(timer_start);
									tempEvent.timerCount++;
								}
								else
								{
									tempEvent.otherTimers.push(tempTimer);
									tempEvent.otherTimers_start[tempEvent.otherTimerCount] = new Date(timer_start);
									tempEvent.otherTimerCount++;
								}

								//tempEvent.min_start_time = min_start_time;
								//tempEvent.max_end_time = max_end_time;

								//timerBar = timerBar + '<div style="color:red" class="epgwiz-evt-timer"> Matching timer: \'' + this.filename + '\'</div>';
							}

							if((tempTimer.reservationtype() !== 0) && (timer_end < max_epg_date))  //This is not an OT timer and we have not passed the EPG end date
							{
								//1 = Daily, 2 = Weekend, 3 = Weekly, 4 = Weekday

								//if (window.console) console.log("Extending timer (" + tempTimer.reservationtype() + ") '" + tempTimer.filename + "' timer_end '" + timer_end + "' max_epg_date '" + max_epg_date + "'");
								extendFlag = 0;
								extended = 0;

								//console.log("A min_start " + min_start_time);

								var skipPattern = [1,1,1,1,1,1,1];
								var skipSize = 1;

								switch(tempTimer.reservationtype())
								{
									case 1: //Daily
										//if (window.console) console.log("Changing daily");
										//console.log("B min_start " + min_start_time);
										timer_end.setDate(timer_end.getDate() + 1);
										timer_start.setDate(timer_start.getDate() + 1);
										//extendFlag = 1;
										//extended = 1;
										break;

									case 3: //Weekly
										//if (window.console) console.log("Changing weekly");
										//console.log("C min_start " + min_start_time);
										timer_end.setDate(timer_end.getDate() + 7);
										timer_start.setDate(timer_start.getDate() + 7);
										//extendFlag = 1;
										//extended = 1;
										break;

									case 2: //Weekend
										//if (window.console) console.log("Changing weekend");
										//console.log("D1 min_start " + min_start_time);
										//Advance 1 day

										//console.log("D2 min_start " + min_start_time);
										skipPattern = [6,5,4,3,2,1,1];
										//console.log("D3 min_start " + min_start_time);
										skipSize = skipPattern[timer_start.getDay()];
										//console.log("D4 min_start " + min_start_time);
										timer_end.setDate(timer_end.getDate() + skipSize);
										//console.log("D5 min_start " + min_start_time + " " + skipSize);
										timer_start.setDate(timer_start.getDate() + skipSize);
										//console.log("D6 min_start " + min_start_time + " " + skipSize);

										//timer_end.setDate(timer_end.getDate() + 1);
										//timer_start.setDate(timer_start.getDate() + 1);
										//var wd = timer_start.getDay();

										//If it is not a weekend day, loop until we find a weekend day.
										//while(wd !== 0 && wd !== 6)	//Looping is not the most efficient method, but it is the most expedient method.
										//{
										//	timer_end.setDate(timer_end.getDate() + 1);
										//	timer_start.setDate(timer_start.getDate() + 1);
										//	wd = timer_start.getDay();
										//}

										//If we have not gone past the maximum date, proceed.
										//if(timer_end < max_epg_date)
										//{
										//	extendFlag = 1;
										//	extended = 1;
										//}
										break;

									case 4: //Weekday
										//if (window.console) console.log("Changing weekday");
										//console.log("E min_start " + min_start_time);

										skipPattern = [1,1,1,1,1,3,2];
										skipSize = skipPattern[timer_start.getDay()];

										//console.log("Skipping " + skipSize.toString() + " from " + timer_start + " " + timer_start.getDay() + " " + timer_end + " " + timer_end.getDay());

										timer_end.setDate(timer_end.getDate() + skipSize);
										timer_start.setDate(timer_start.getDate() + skipSize);

										//console.log("  got " + timer_start + " " + timer_start.getDay() + " " + timer_end + " " + timer_end.getDay());

										break;

									default:
										//If we have encountered an un-handled reservation type, force and exit from the loop.
										//if (window.console) console.log("Changing default");
										//console.log("F min_start " + min_start_time);
										timer_end.setDate(max_epg_date.getDate() + 30);
										extendFlag = 0;
										extended = 0;
										break;

								}//END switch on reservation type

								if(timer_start <= max_epg_date)
								{
									//console.log("G min_start " + min_start_time);
									//if (window.console) console.log("New timer_end '" + timer_end + " max_epg_date '" + max_epg_date);
									extendFlag = 1;
									extended = 1;
								}

								//console.log("H min_start " + min_start_time);
								//if (window.console) console.log("New timer_start '" + timer_start + " New timer_end '" + timer_end + " " + extendFlag + " " + extended);

							}//END test reservation type and date match
							//console.log("I min_start " + min_start_time);
						}//END extended flag loop
						//console.log("J min_start " + min_start_time);
					//}//END logoid test
				});

				//Find any recordings-in-progress
				tempEvent.recordingsCount = 0;
				tempEvent.recordings = []; //new Array();
				$.each(tma.status.recording, function(i, module) {

					tempRec = this;

					//if(tempRec.logoid == tempEvent.logoid) //Is this the same channel?
					if(tempRec.rectype === 1)
					{
						var rec_start = new Date(tempRec.starttime);
						var rec_end = new Date(tempRec.endtime);

						if( (rec_end   >= event_start_date && rec_end <= event_end_date) ||
							(rec_start <= event_end_date   && rec_end >= event_end_date) ||
							(rec_start >= event_start_date && rec_end <= event_end_date) ||
							(rec_start <= event_start_date && rec_end >= event_end_date) )
						{
							//if (window.console) console.log("Active recording '" + tempRec.filename + "' matches event '" + tempEvent.name + "'");

							if(rec_start < min_start_time){min_start_time = new Date(rec_start);}
							if(rec_end > max_end_time){max_end_time = new Date(rec_end);}

							tempEvent.recordings.push(tempRec);
							tempEvent.recordingsCount++;
						}
					}
				});

				//if (window.console) console.log("Matched " + tempEvent.timerCount.toString() + " timers.");

				if(tempEvent.timerCount !== 0 || tempEvent.otherTimerCount !== 0 || tempEvent.recordingsCount !== 0)
				{
					//Build the event & timer blocks
					var blockDuration = self.calcDuration(max_end_time, min_start_time);
					var eventDuration = self.calcDuration(event_end_date, event_start_date);
					//console.log("max_end_time " + max_end_time + " min_start_time " + min_start_time);
					//console.log("event_end_date " + event_end_date + " event_start_date " + event_start_date);
					//console.log("blockDuration " + blockDuration + " eventDuration " + eventDuration);

					//if (window.console) console.log("min_start_time " + min_start_time.toString() + " max_end_time " + max_end_time.toString());
					//if (window.console) console.log("event_start_date " + event_start_date.toString() + " event_end_date " + event_end_date.toString());

					var spacerPercent = 0;
					if(min_start_time < event_start_date)
					{
						spacerPercent = (self.calcDuration(event_start_date, min_start_time) * 100) / blockDuration;
					}
					var eventPercent = (eventDuration * 100) / blockDuration;

					//Display the EPG event block
					var timesBlock = tma.date('H:i', event_start_date) + " - " + tma.date('H:i', event_end_date);
					timerBar = timerBar +
						'<div class="epgwiz-evt-timers">' +
						'	<div class="epgwiz-evt-evt-block">' +
						'		<div title="' + timesBlock + '" class="epgwiz-evt-evt" style="left:' + spacerPercent.toString() + '%; width: ' + eventPercent.toString() + '%;"></div>' +
						'	</div>';

					if(tempEvent.timerCount > 0)
					{
						timerBar = timerBar +
							'	<div class="epgwiz-evt-timers-block">';
					}

					//Display each matching timer
					var t_start, timerMulti, timerExtended, timerRecClass;
					var durationPercent = 0; var startPercent = 0;
					for(i = 0; i < tempEvent.timerCount; i++)
					{
						//var t_start = new Date(tempEvent.timers[i].starttime);
						t_start = new Date(tempEvent.timers_start[i]);
						durationPercent = (tempEvent.timers[i].duration * 100) / blockDuration;
						//console.log("min_start_time " + min_start_time + " t_start " + t_start + " duration " + self.calcDuration(t_start, min_start_time) + " " + tempEvent.timers[i].starttime);
						startPercent = (min_start_time < t_start) ? (self.calcDuration(t_start, min_start_time) * 100) / blockDuration : 0;
						timesBlock = tma.date('H:i', tempEvent.timers[i].starttime) + " - " + tma.date('H:i', tempEvent.timers[i].endtime) + " (" + tempEvent.timers[i].filename.slice(0,-4) + ")";
						timerMulti = (tempEvent.timerCount > 1) ? "-multi" : "";
						timerExtended = (tempEvent.timers[i].extended) ? "extended" : "timers";
						//var timerRecClass = (tempEvent.timers[i].isrec == 1) ? "epgwiz-evt-timers-rec" + timerMulti : "epgwiz-evt-timers-play" + timerMulti;
						timerRecClass = (tempEvent.timers[i].isrec == 1) ? "epgwiz-evt-" + timerExtended + "-rec" + timerMulti : "epgwiz-evt-" + timerExtended + "-play" + timerMulti;

						timerBar = timerBar +
							'<div title="' + timesBlock + '" class="' + timerRecClass + '" style="left:' + startPercent.toString() + '%; width: ' + durationPercent.toString() + '%;"></div>';

					}//End loop through timers attached to this event.

					if(tempEvent.timerCount > 0)
					{
						timerBar = timerBar +
						'	</div>';
					}

					for(i = 0; i < tempEvent.otherTimerCount; i++)
					{
						timerBar = timerBar +
							'	<div class="epgwiz-evt-timers-block">';

						//var t_start = new Date(tempEvent.timers[i].starttime);
						t_start = new Date(tempEvent.otherTimers_start[i]);
						durationPercent = (tempEvent.otherTimers[i].duration * 100) / blockDuration;
						//console.log("min_start_time " + min_start_time + " t_start " + t_start + " duration " + self.calcDuration(t_start, min_start_time) + " " + tempEvent.otherTimers[i].starttime);
						startPercent = (min_start_time < t_start) ? (self.calcDuration(t_start, min_start_time) * 100) / blockDuration : 0;
						timesBlock = tma.date('H:i', tempEvent.otherTimers[i].starttime) + " - " + tma.date('H:i', tempEvent.otherTimers[i].endtime) + " (" + tempEvent.otherTimers[i].filename.slice(0,-4) + " | " + tempEvent.otherTimers[i].chname + ")";
						timerMulti = "-other";
						timerExtended = (tempEvent.otherTimers[i].extended) ? "extended" : "timers";
						timerRecClass = (tempEvent.otherTimers[i].isrec == 1) ? "epgwiz-evt-" + timerExtended + "-rec" + timerMulti : "epgwiz-evt-" + timerExtended + "-play" + timerMulti;

						timerBar = timerBar +
							'<div title="' + timesBlock + '" class="' + timerRecClass + '" style="left:' + startPercent.toString() + '%; width: ' + durationPercent.toString() + '%;"></div>';

						timerBar = timerBar +
						'	</div>';


					}//End loop through other timers attached to this event.

					//Display each matching recording-in-progress
					for(i = 0; i < tempEvent.recordingsCount; i++)
					{
						timerBar = timerBar +
							'	<div class="epgwiz-evt-timers-block">';

						var rec_start = new Date(tempEvent.recordings[i].starttime);
						var channel = tma.channel(tempEvent.recordings[i].logoid);
						durationPercent = (tempEvent.recordings[i].duration * 100) / blockDuration;
						startPercent = (min_start_time < rec_start) ? (self.calcDuration(rec_start, min_start_time) * 100) / blockDuration : 0;
						timesBlock = tma.date('H:i', tempEvent.recordings[i].starttime) + " - " + tma.date('H:i', tempEvent.recordings[i].endtime) + " (" + tempEvent.recordings[i].filename.slice(0,-4) + " | " + channel.servicename + ")";

						//if(tempEvent.timerCount !== 0)
						//{
						//	timerBar = timerBar +
						//		'	</div>' +
						//		'	<div class="epgwiz-evt-timers-block">' ;
						//}

						timerBar = timerBar +
							'<div title="' + timesBlock + '" class="epgwiz-evt-rec-progress" style="left:' + startPercent.toString() + '%; width: ' + durationPercent.toString() + '%;"></div>';

							timerBar = timerBar +
						'	</div>';

					}//End loop through active recordings.

					//Close the DIV
					timerBar = timerBar +
						'	</div>' +
						'</div>';
				}

				//If this a event current, show the progress bar.
				var currBarTmpl = "";
				if ((queryStartTime <= max_end_time) && (queryStartTime >= min_start_time))
				{
					var percent = parseInt((queryStartTime - min_start_time) * 100 / (event_end_date - min_start_time));

					currBarTmpl =		'<span class="epgwiz-evt-curr">' +
											'<span class="epgwiz-evt-curr-part" style="width: '+ percent.toString() +'%;" />' +
										'</span>';
				}

				var currentEvt = evtTmpl + currBarTmpl + timerBar + evtTmp2;
				//if (window.console) console.log("Service name '" + svcName + " logoid " + this.logoid);

				//var $event = $( evtTmpl.format({
				var $event = $( currentEvt.format({
					eventid:		this.eventid,
					timestamp:		(new Date(this.starttime_local)).getTime(),
					date:			tma.date(self.i18.shortdate_fmt, this.starttime_local),
					start:			tma.date('H:i', this.starttime_local),
					end:			tma.date('H:i', this.endtime_local),
					duration:		this.duration,
					dur_fmt: 		tma.formatDuration(this.duration),
					name:			this.name,
					//genre:			getGenreAndParentalString(this, prefs.getGenreColoring() ),
					//genre:			getGenreAndParentalString(tempEvent, parseInt( self.localSetting( 'genre-coloring', 1 ))),
					genre:			getGenreAndParentalString(tempEvent, 0),
					description:	this.description,
					hasdescr:		(this.description) ? 'y' : 'n',
					text:			this.htmlText(),
					logoid:			this.logoid,
					haslogo:		(this.channel.logo !== null),
					servicename:	this.channel.servicename,
					//text:			this.text,
				}) )

				.prop('event', tempEvent)
				.on('click', function(e) {

					var $prg = $(this);

					// check, if any module handles the 'showEvent' event
					// and prevent reopening of the same info
					if (
							(tma.fn.showEvent) &&
							!$prg.hasClass('jk-mark')
					) {

						var event = $prg.prop('event');

						tma.fn.showEvent( event ).done(function(popup) {
							popup.$relatedPrg = $prg;
							$(popup).on('close', function() {
								this.$relatedPrg.removeClass('jk-mark');
							});
						});
						$prg.addClass('jk-mark');
					}
				})
				//.appendTo($resultsTable);
				.appendTo($tbody);

			});

			if(foundItems != results.count)  //Some found items were filtered because they were in the past.
			{
				if(foundItems == 1)
				{
					hdr = self.i18.results_found_single;
				}
				else
				{
					hdr = self.i18.results_found_plural;
				}

				var $tcount = $resultsTable.find('#epgwiz-results-count').html(foundItems.toString() + '&nbsp' + hdr);
			}

			$resultsTable.appendTo($result);
		}
		else
		{
			self.displayNoMatch();
		}

	};//END presentResults()

	this.initepgsearchTAP = function () {

		var params = {
				'function'	: 'epg',
				action		: 'search!',
				//tap			: this.TapId + '!',
				//epgsource	: epgsource,
				cache		: '1'		//If cache is set to '1', the TAP will not delete the EPG cache unless the timeout has elapsed
		};

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		$.getJSON('/api', params).done(function(data, status, xhr) {

			//alert("TAP returned port: " + data.port);
			//if (window.console) console.log("TAP returned port: " + data.port);

			this.port = data.port;
			searchPort = data.port;
			error_code = data.error_code;
			queryStartTime = new Date(data.timestamp);

			if (data.content === "epgsearch_ack")
			{
				//if (data.error_code === 0)
				//{
					deferred.resolve(data, status, xhr);
				//}
				//else
				//{
				//	self.displayTAPBusy();
				//}
			}
			else
			{
				// switch to fail()
				// TODO think about standardised errors
				deferred.reject(
						xhr,
						'tma-error',
						//'epgsearch: ' + data.status_text
						'epgsearch: ' + data.timestamp
				);
			}

		}).fail(function(xhr, errorType, error) {
			deferred.reject(xhr, errorType, error);
		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in epgsearch.initepgsearchTAP()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
				}
			});

		});

	};

	this.displayPleaseWait = function() {

		//var result = '<img id="epgwiz-wait" title="Please Wait" src="css/img/wait.gif">Searching EPG, please wait.';
		//var result = '<img id="epgwiz-wait" title="Please Wait" src="modules/epgsearch/img/pleasewait.gif"><span id="epgwiz-progress-text">Searching EPG, please wait.</span>';
		var result = '<img id="epgwiz-wait" title="Please Wait" src="css/img/wait.gif"><span id="epgwiz-progress-text">' + self.i18.wait_searching + '</span>';

		result = result + '<div id="epgwiz-progress-progress" class="epgwiz-progress-graph"><div id="epgwiz-progress-bar" style="width:0%"></div>';

		$('DIV.epgwiz DIV.epgwiz-results').html(result);

	};//END displayPleaseWait()

	this.displayNoMatch = function() {

		var result = self.i18.no_matches;

		$('DIV.epgwiz DIV.epgwiz-results').html(result);

	};//END displayNoMatch()

	this.displayTAPBusy = function() {

		var result = self.i18.tap_busy;

		$('DIV.epgwiz DIV.epgwiz-results').html(result);

	};//END displayTAPBusy()

	this.displayQueryFailed = function() {

		var result = self.i18.search_failed;

		$('DIV.epgwiz DIV.epgwiz-results').html(result);

	};//END displayQueryFailed()

	this.calcDuration = function(t1, t2) {

		return (t1.getTime() - t2.getTime()) / 60000;

	};//END calcDuration()

	this.getFavDesc = function(favGroup) {

		var favDesc = "";

		switch (favGroup)
		{
			case '':
				favDesc = "";
				break;
			case '__tv':
				favDesc = "(" + tma.i18.favgroups.tv + ")";
				break;
			case '__radio':
				favDesc = "(" + tma.i18.favgroups.radio + ")";
				break;
			default:
				favDesc = "(" + favGroup + ")";
		}

		return favDesc;

	};//END getFavDesc()

	this.executeQuery = function(queryText, port) {

		//Stop the timer from triggering again.
		if (self.delayQuery) {
			window.clearTimeout(self.delayQuery);
			self.delayQuery = null;
		}

		var url = 'ws://' + window.location.hostname + ':' + port + '/epgsearch';

		//alert("URL = " + url);

		//alert("{" + queryText + "}");

		this.displayPleaseWait();

		if ('WebSocket' in window)
		{
			socket = new WebSocket(url);
			messageBuffer = "";

			socket.onopen = function(msg) {
				if(socket.readyState != WebSocket.OPEN)
				{alert("Socket not ready to send");}
				socket.send(queryText);
			};

			socket.onmessage = function(msg) {
				//alert(msg.data);

				var magic = msg.data.slice(0, 12);

				//alert("'"+magic+"'");

				if(magic == "@@@@FATAL@@@")
				{
					//$('#epgwiz-progress-text').html("Fatal error searching");
					$('DIV.epgwiz DIV.epgwiz-results').html(self.i18.fatal_error);
				}
				else if(magic != "@@PROGRESS@@")
				{
					//messageBuffer = messageBuffer + msg.data.toString();
					messageBuffer = messageBuffer + msg.data;
				}
				else
				{
					//alert(msg.data.toString());
					var prgss = msg.data.toString();
					//alert(prgss);
					//self.displayProgress(prgss);

					$('#epgwiz-progress-text').html(self.i18.percent_progress1 + prgss.slice(13) + self.i18.percent_progress2);
					$('div#epgwiz-progress-bar').width(prgss.slice(13)+'%');
				}

			};
			socket.onclose = function(msg) {

				if(msg.code != 1000 && msg.code != 1006) //1000 = normal close, 1006 = socket closed without closing WS frame.
				{
					alert("WebSocket closed with code " + msg.code.toString());
				}

				//alert(messageBuffer.length.toString() + " [" + messageBuffer + "]");
				if(messageBuffer.length > 0)
				{
					//alert(messageBuffer.length.toString() + " [" + messageBuffer + "]");
					responseObject = JSON.parse(messageBuffer);

					//If there are no matches, show the no match error, otherwise, update the current status (looking for recordings in progress)
					if(responseObject.count === 0)
					{
						self.displayNoMatch();
					}
					else
					{

						// jk: convert plain JS objects to TmaEpgEvent objects and add channel
						$.each( responseObject.event, function(i, event) {
							responseObject.event[i] = new TmaEpgEvent(event);
						} );

						tma.apiGetStatus().done(function(status) {
							self.presentResults(responseObject);
						});
					}
				}
				//alert(responseObject.content + " - " + responseObject.count);
				socket = null;
			};

			socket.onerror = function(msg) {
				//TODO - A lot of these error occur because TMAapprentice is not yet listening to the port.
				//TODO - Build a retry loop that tries every 500ms for 5 seconds before displaying a fatal error message.
				//alert("Error: " + msg.data);
				//alert("Error: " + msg.code); //The code is not available for this event.
				self.displayQueryFailed();
			};
		}
		else
		{
			alert(self.i18.no_websockets);
		}

	};//END executeQuery()

	this.loadSearchFields = function(searchDefinition) {

		//Separate function to load search field so that future SE search imports can be processed with this function.

		//alert(searchDefinition.name);
		//if (window.console) {console.log("Loading search '" + searchDefinition.name + "'.");}

		//Load the favourites group and update the channels list for that group
		favGroup = searchDefinition.favgroup;
		//var chList = tma.getChannels(favGroup);

		//newList = "";
		//$.each(chList, function(ix, channel) {
		//	newList = newList + '<option value="' + channel.logoid + '">' + channel.servicename + '</option>';
		//});

		//$('#searchservice')
		//	.empty()
		//	.append(newList)
		//	.change();
		//
		self.fieldSet_services(searchDefinition.favgroup);

		//Read through the selected channels for this search and select each channel <option> in the <select> with a matching logoid
/* 		if(searchDefinition.channelcount > 0)
		{
			for(var i = 0; i < searchDefinition.channelcount; i++)
			{
				var opt = 'option[value="' + searchDefinition.channels[i] + '"]';
				$('#searchservice ' + opt).attr('selected',true);
			}
		}
 */
		$.each(searchDefinition.channels, function(i, chObject){

			var opt = 'option[value="' + chObject.logoid + '"]';
			$('#searchservice ' + opt).attr('selected',true);

		});

		//Force and update to the text descriptor
		$('#searchservice').change();

		//Update the criteria heading to include the same of the search
		$('#wpgwiz-saved-search-name').html(self.i18.text_criteria + " [" + searchDefinition.name +"]");
		currentSearchName = searchDefinition.name;

		//Load the date, time, duration fields
		//$('#minlen').timepicker({'timeFormat' : 'H:i'});
		//$('#minlen').timepicker('setTime', searchDefinition.minlen);
		//$("#minlentextblock").text(self.durationFormatter(searchDefinition.minlen));

		self.fieldSet_time('minlen', searchDefinition.minlen);

		//$('#maxlen').timepicker('setTime', searchDefinition.maxlen);
		//$("#maxlentextblock").text(self.durationFormatter(searchDefinition.maxlen));
		self.fieldSet_time('maxlen', searchDefinition.maxlen);

		//$('#starttime').timepicker('setTime', searchDefinition.starttime);
		//$("#starttimetextblock").text(self.timeFormatter(searchDefinition.starttime));

		self.fieldSet_time('starttime', searchDefinition.starttime);

		//$('#endtime').timepicker('setTime', searchDefinition.endtime);
		//$("#endtimetextblock").text(self.timeFormatter(searchDefinition.endtime));

		self.fieldSet_time('endtime', searchDefinition.endtime);

		//Load the dates.  Empty dates were seen as 'today' so need to be handled specially.
/* 		if(searchDefinition.startdate == '')
		{
			$('#startdate').datetimepicker('setDate', null);
			$('#startdate').val("");
			$('#startdatetextblock').text(self.i18.not_limited);
		}
		else
		{
			$('#startdate').datetimepicker('setDate', searchDefinition.startdate);
			$("#startdatetextblock").text(tma.date(self.i18.timerdate_fmt, searchDefinition.startdate));
		}
 */		self.fieldSet_date('startdate', searchDefinition.startdate);


/* 		if(searchDefinition.enddate == '')
		{
			$('#enddate').datetimepicker('setDate', null);
			$('#enddate').val("");
			$('#enddatetextblock').text(self.i18.not_limited);
		}
		else
		{
			$('#enddate').datetimepicker('setDate', searchDefinition.enddate);
			$("#enddatetextblock").text(tma.date(self.i18.timerdate_fmt, searchDefinition.enddate));
		}
 */
		self.fieldSet_date('enddate', searchDefinition.enddate);

		//Load the weekdays
		$('LABEL.epgwiz-weekday').epgwizWeekday(searchDefinition.weekday[1], searchDefinition.weekday[2], searchDefinition.weekday[3], searchDefinition.weekday[4], searchDefinition.weekday[5], searchDefinition.weekday[6], searchDefinition.weekday[7]);


		//self.addTextRow(defaultValues)

		$('#text-criteria-table >tbody >tr').remove();

/* 		if(searchDefinition.textcount > 0)
		{
			for(var i = 0; i < searchDefinition.textcount; i++)
			{
				self.addTextRow(searchDefinition.texts[i]);
			}
		}
		else
		{
			self.addTextRow(null);
		}
 */

		var i = 0;

		$.each(searchDefinition.texts, function(ind, textObject){
			self.addTextRow(textObject);
			i++;
		});

		if(i === 0)
		{
			self.addTextRow(null);
		}
	};

	this.findSearchFileName = function(searchName, callback) {
		//Take the search name, lookup the index file and return the file name.  If the search name is not found, return a serial number to be used as the name.

		var d = new Date();
		var serialName = d.getFullYear().toString() + ("0" + (d.getMonth() + 1).toString()).slice(-2) + ("0" + (d.getDate()).toString()).slice(-2) + ("0" + (d.getHours()).toString()).slice(-2) + ("0" + (d.getMinutes()).toString()).slice(-2) + ("0" + (d.getSeconds()).toString()).slice(-2);

		var nameObject = {fileName: serialName, newSearch: 1, searchName: searchName};

		$.getJSON(indexFileName, null, function(fileIndexObject){

			$.each(fileIndexObject, function(obj, objData){
				//if (window.console) {console.log("object = '" + obj + "'");}
				//if (window.console) {console.log("matching '" + searchName + "' against - searchName = '" + fileIndexObject[obj].searchName + "' fileName = '" + fileIndexObject[obj].fileName + "'");}
				//if (window.console) {console.log("matching '" + searchName + "' against - searchName = '" + objData.searchName + "' fileName = '" + objData.fileName + "'");}

				if(objData.searchName == searchName)
				{
					//if (window.console) {console.log("Matched");}
					nameObject.newSearch = 0;
					nameObject.fileName = objData.fileName;

					//if (window.console) {console.log("Returning " + JSON.stringify(nameObject));}
					//callback(nameObject);

					return false;
				}
			});

			//if (window.console) {console.log("Returning " + JSON.stringify(nameObject));}
			callback(nameObject);

		}).fail(function(xhr, errorType, error) {

			if(xhr.status == 404)
			{
				//if (window.console) {console.log("Index file not found " + JSON.stringify(nameObject));}
				callback(nameObject);
			}
			else
			{
				tma.log({
					title: errorType + " in epgsearch.loadSearchDefinition()",
					details: {
						error: error,
						xhrReadyState: xhr.readyState,
						xhrStatus: xhr.status,
					}
				});
				callback(null);
			}
		});
	}; //END findSearchFileName

	this.saveFile = function(fileName, fileContents) {

		//if (window.console) {console.log("Saving file '" + fileName + "'");}

		$.post('/api?function=files&action=store!&filename=' + fileName + '&', fileContents, null, 'json').done(function(data) {
			//if (window.console) {console.log("saveFile() 5");}
			if (data.error_code !== 0) {
				tma.log({
					title: "tma-error in epgsearch.saveFile()",
					details: {
						status: data.status,
						parameter: 'save file ' + data.error_text,
						errno: data.errno.toString(),
						fileName: fileName,
					}
				});
			}
		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in epgsearch.saveFile()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: 'fail',
				}
			});

		});
	};//END saveFile()

	this.updateSearchIndex = function(searchObject, deleteFlag) {

		var indexObject = []; //new Array(); //{searches: []}
		//if (window.console) {console.log("updateSearchIndex() searchName: '" + searchObject.searchName + "' fileName: '" + searchObject.fileName + "' " + deleteFlag);}

		$.getJSON(indexFileName, function(fileIndexObject){

			//if (window.console) {console.log("updateSearchIndex() 1 " + JSON.stringify(fileIndexObject, null, '\t'));}
			//alert("READ " + JSON.stringify(fileIndexObject, null, '\t'));
			//self.loadSearchFields(indexObject);
			//indexObject.push(fileIndexObject);

			if(deleteFlag)
			{
				$.each(fileIndexObject, function(i, indexObject){

					//if (window.console) {console.log(searchObject.searchName + " : " + indexObject.searchName + " / " + searchObject.fileName + " : " + indexObject.fileName);}

					if(searchObject.searchName == indexObject.searchName && searchObject.fileName == indexObject.fileName)
					{
						//if (window.console) {console.log("Match");}
						fileIndexObject.splice(i,1);
					}
				});
			}
			else
			{
				//$.extend(indexObject, fileIndexObject, {searchName: searchObject.searchName, fileName: searchObject.fileName});
				fileIndexObject.push({searchName: searchObject.searchName, fileName: searchObject.fileName});
			}

			//alert("POST-READ " + JSON.stringify(indexObject, null, '\t'));

			//if (window.console) {console.log("updateSearchIndex() 2 " + JSON.stringify(fileIndexObject, null, '\t'));}

			//indexObject.push({searchName: searchObject.searchName, fileName: searchObject.fileName});

			self.saveFile(indexFileName, JSON.stringify(fileIndexObject, null, '\t'));


		}).done(function() {
			//if (window.console) {console.log("updateSearchIndex() done");}
		}).fail(function(xhr, errorType, error) {
			//if (window.console) {console.log("updateSearchIndex() fail");}
			if(xhr.status == 404)
			{
				//$.extend(indexObject, {searchName: searchObject.searchName, fileName: searchObject.fileName});
				indexObject.push({searchName: searchObject.searchName, fileName: searchObject.fileName});
				self.saveFile(indexFileName, JSON.stringify(indexObject, null, '\t'));
				//indexObject = [];
			}
			else
			{
				tma.log({
					title: errorType + " in epgsearch.updateSearchIndex()",
					details: {
						error: error,
						xhrReadyState: xhr.readyState,
						xhrStatus: xhr.status,
					}
				});
				return 0;
			}
		});

		//if (window.console) {console.log("updateSearchIndex() 3");}
		//alert("PRE-PUSH " + JSON.stringify(indexObject, null, '\t'));

		//indexObject.push({searchName: searchObject.searchName, fileName: searchObject.fileName});

		//alert("POST-PUSH " + JSON.stringify(indexObject, null, '\t'));
		//if (window.console) {console.log("updateSearchIndex() 4");}

/* 		$.post('/api?function=files&action=store!&filename=' + indexFileName + '&', JSON.stringify(indexObject, null, '\t'), null, 'json').done(function(data) {
			if (window.console) {console.log("updateSearchIndex() 5");}
			if (data.error_code !== 0) {
				tma.log({
					title: "tma-error in epgsearch.updateSearchIndex()",
					details: {
						status: data.status,
						parameter: 'save file name ' + data.error_text,
						errno: data.errno.toString(),
					}
				});
			}
		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in epgsearch.updateSearchIndex()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: 'fail',
				}
			});

		}); */


	}; //END updateSearchIndex


	this.loadSearchDefinition = function(searchName) {

		//Load the search definition from file and then pass it to the loadSearchFields() function.
/* 		if(searchFileName === undefined)
		{
			searchFileName = "/mnt/hd/ProgramFiles/Settings/WebControl/public_html/modules/epgsearch/searches/test01.json";

			$('#wpgwiz-saved-search-name').html(self.i18.text_criteria + " [test01]");
			//searchDefinition.name
		}
		else
		{
			searchFileName = "/mnt/hd/ProgramFiles/Settings/WebControl/public_html/modules/epgsearch/searches/" + searchFileName;
			$('#wpgwiz-saved-search-name').html(self.i18.text_criteria + " [" + searchFileName +"]");
		}
 */

		self.findSearchFileName(searchName, function(loadFile){

			if (!loadFile.newSearch)
			{
				var searchFileName = PVRdirectory + "/searches/" + loadFile.fileName + "_search.json";

				$.getJSON(searchFileName, function(searchDefinition){

					self.loadSearchFields(searchDefinition);

				}).fail(function(xhr, errorType, error) {

					if(xhr.status == 404)
					{
						alert(self.i18.saved_search_file + " " + searchName + " " + self.i18.not_found);  //This should be a rare error so an 'alert' is OK.
					}
					else
					{
						tma.log({
							title: errorType + " in epgsearch.loadSearchDefinition()",
							details: {
								error: error,
								xhrReadyState: xhr.readyState,
								xhrStatus: xhr.status,
							}
						});
					}
				});
			}
			else
			{
				alert(self.i18.saved_search_def + " " + searchName + " " + self.i18.not_found);  //This should be a rare error so an 'alert' is OK.
			}
		});
	};//END loadSearchDefinition()

	this.deleteSearchDefinition = function(searchFileName, searchDefinition) {

		//if (window.console) {console.log("deleteSearchDefinition() '" + searchFileName + "' '" + searchDefinition + "'");}

		//var delSearchObject = new Array();

		//Try to delete the search file first
		var params =
		"file="	+ searchFileName + "_search.json" + "\n" +
		"dir="	+ PVRdirectory + "/searches" + "\n";

		$.post('/api?function=files&action=delete!', params, null, 'json').done(function(data) {

		//tma.apiDeleteFile(PVRdirectory + "/searches", searchFileName + "_search.json", "").done(function(data) {

				//if (window.console) {console.log(data.mpg_status + " " + data.inf_status + " " + data.nav_status === 0);}
				//If the search file has been deleted, delete the index record.
				if ((data.mpg_status === 0) && (data.inf_status === 0) && (data.nav_status === 0))
				{
					//Delete the index record here
					//if (window.console) {console.log("Definition file deleted");}
					//delSearchObject.push({searchName: searchDefinition, fileName: searchFileName, newSearch: 0});
					//if (window.console) {console.log("Object created");}
					//self.updateSearchIndex(delSearchObject, true);
					self.updateSearchIndex({searchName: searchDefinition, fileName: searchFileName, newSearch: 0}, true);
					//if (window.console) {console.log("Index updated");}
				} else {
					alert("Error deleting definition file '" + searchDefinition + "' '" + searchFileName + "'");
				}

			});

	};//END deleteSearchDefinition()

	this.saveSearchFile = function(saveFile, searchDefinition) {

		var searchFileName = PVRdirectory + "/searches/" + saveFile.fileName + "_search.json";

		//jQuery.post( url [, data ] [, success ] [, dataType ] )
		$.post('/api?function=files&action=store!&filename=' + searchFileName + '&', JSON.stringify(searchDefinition, null, '\t'), null, 'json').done(function(data) {

			if (data.error_code !== 0) {
				tma.log({
					title: "tma-error in epgsearch.saveSearchFile()",
					details: {
						status: data.status,
						parameter: 'save ' + data.error_text,
						errno: data.errno.toString(),
					}
				});
			}
			else
			{
				if(saveFile.newSearch)  //TODO - Should this be here?  savedFile could be undefined
				{
					self.updateSearchIndex(saveFile, false);

					$('#wpgwiz-saved-search-name').html(self.i18.text_criteria + " [" + saveFile.searchName +"]");
					currentSearchName = saveFile.searchName;
				}
			}

		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in epgsearch.saveSearchFile()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: 'fail',
				}
			});

		});


	};

	this.saveSearchDefinition = function(searchName) {

		//Take the search fields and create a JS Object to be saved

		if(searchName === undefined || searchName === '')
		{

			searchName = prompt("Please enter a search name", "New EPG Search");

			if(searchName === null || searchName === undefined)
			{
				searchName = "Unnamed EPG Search";
			}
		}

		var searchDefinition = {
			name:		searchName,
			minlen:		$('#minlen').val(),
			maxlen:		$('#maxlen').val(),
			startdate:	$('#startdate').val(),
			enddate:	$('#enddate').val(),
			starttime:	$('#starttime').val(),
			endtime:	$('#endtime').val(),

			favgroup:	favGroup,

			weekday:	{},

			texts:		[], //new Array(), // jk: I would recommend an array
			//textcount:	0,	//     (would then be obsolete)

			channels:	[],  //new Array(), // jk: I would recommend an array
			//channelcount:0,	//     (would then be obsolete)

		};

		for(var i = 1; i < 8; i++)
		{
			searchDefinition.weekday[i] = $('#epgwiz-wd' + i.toString()).prop('checked');
		}

		$('#text-criteria-table tbody tr').each(function(){

			searchDefinition.texts.push({

				text:	$(this).find('#epgwiz-search-text').val(),

				title:	$(this).find('#epgwiz-search-title').prop('checked'),
				desc:	$(this).find('#epgwiz-search-desc').prop('checked'),
				ext:	$(this).find('#epgwiz-search-ext').prop('checked'),

				points:	$(this).find('#epgwiz-search-points').val(),
			});

		});//END loop through text criteria

		var $selChannels = $('#searchservice :selected').each(function() {

			searchDefinition.channels.push({

				logoid:	$(this).attr('value')

			});

		});

		//Because findSearchFileName() is asynchronous, I need to setup a function to be called once findSearchFileName() is complete.
		self.findSearchFileName(searchName, function(saveFile){

			//var searchFileName = PVRdirectory + "/searches/" + saveFile.fileName + "_search.json";

			if(saveFile.newSearch === 0)
			{
				new jkqDialog({
					title: self.i18.replace,
					text: self.i18.replace_existing + " '" + searchName + "'",
					buttons: {
						ok:				 tma.i18.respond_yes,
						jkqDialogCancel: tma.i18.respond_cancel
					}
				}).confirm(function(button) {

					if (button == 'ok') {
						//self.saveSearchFile(searchFileName, searchDefinition);
						self.saveSearchFile(saveFile, searchDefinition);
					}
				});
			}
			else
			{
				//self.saveSearchFile(searchFileName, searchDefinition);
				self.saveSearchFile(saveFile, searchDefinition);
			}



			// //jQuery.post( url [, data ] [, success ] [, dataType ] )
			// $.post('/api?function=files&action=store!&filename=' + searchFileName + '&', JSON.stringify(searchDefinition, null, '\t'), null, 'json').done(function(data) {

				// if (data.error_code !== 0) {
					// tma.log({
						// title: "tma-error in epgsearch.saveSearchDefinition()",
						// details: {
							// status: data.status,
							// parameter: 'save ' + data.error_text,
							// errno: data.errno.toString(),
						// }
					// });
				// }
				// else
				// {
					// if(saveFile.newSearch)
					// {
						// self.updateSearchIndex(saveFile);

						// $('#wpgwiz-saved-search-name').html(self.i18.text_criteria + " [" + searchName +"]");
						// currentSearchName = searchName;
					// }
				// }

			// }).fail(function(xhr, errorType, error) {

				// tma.log({
					// title: errorType + " in epgsearch.saveSearchDefinition()",
					// details: {
						// error: error,
						// xhrReadyState: xhr.readyState,
						// xhrStatus: xhr.status,
						// parameter: 'fail',
					// }
				// });

			// });
		});
	};//END saveSearchDefinition()

	var descriptionsCache = {};

	// help text cache
	var helpText = '';

	/**
	 * reads the module help text
	 */
	this.getModuleHelpText = function() {

		var deferred = $.Deferred();

		if (!helpText) {

			// i18.module.helptext is a filename
			var filename = this.path() + 'lng/' + this.i18.module.helptext;

			// load help text
			$.get(filename).done(function(data) {

				helpText = data;
				deferred.resolve(helpText);

			});

		} else {

			deferred.resolve(helpText);

		}

		return deferred.promise();

	};

	this.resetAllCriteria = function() {
	//Reset all criteria fields

		currentSearchName = "";
		$('#wpgwiz-saved-search-name').html(self.i18.text_criteria);

		self.fieldSet_time('minlen', '');
		self.fieldSet_time('maxlen', '');
		self.fieldSet_time('starttime', '');
		self.fieldSet_time('endtime', '');
		self.fieldSet_date('startdate', '');
		self.fieldSet_date('enddate', '');

		$('LABEL.epgwiz-weekday').epgwizWeekday(true, true, true, true, true, true, true);

		favGroup = tma.localSetting('defaultfavourites', '');
		self.fieldSet_services(favGroup);

		$('#text-criteria-table >tbody >tr').remove();
		self.addTextRow(null);

	};

	this.fieldSet_time = function(fieldName, newValue) {

		if (newValue === '') {newValue = "00:00";}

		$("#" + fieldName).timepicker({'timeFormat' : 'H:i'});
		$("#" + fieldName).timepicker('setTime', newValue);
		$("#" + fieldName).val(newValue);
		$("#" + fieldName + "textblock").text(self.timeFormatter(newValue));
	};//END fieldSet_time()

	this.fieldSet_date = function(fieldName, newValue) {

		if(newValue === '')
		{
			$("#" + fieldName).datetimepicker('setDate', null);
			$("#" + fieldName).val("");
			$("#" + fieldName + "textblock").text(self.i18.not_limited);
		}
		else
		{
			$("#" + fieldName).datetimepicker('setDate', newValue);
			$("#" + fieldName + "textblock").text(tma.date(self.i18.timerdate_fmt, newValue));
		}
	};//END fieldSet_time()


	this.fieldSet_services = function(favGroup) {

		var chList = tma.getChannels(favGroup);

		var newList = "";
		$.each(chList, function(ix, channel) {
			newList = newList + '<option value="' + channel.logoid + '">' + channel.servicename + '</option>';
		});

		$('#searchservice')
			.empty()
			.append(newList)
			.change();

	};//END fieldSet_services()

	this.sortSearchList = function() {

		$('#epgwiz-search-list').html(
			$('.epgwiz-search-item').sort(function(a, b)
			{
				//console.log($(a).attr('sortvalue') + " / " + $(b).attr('sortvalue'));

				if ($(a).attr('sortvalue') < $(b).attr('sortvalue')) return -1;
				if ($(a).attr('sortvalue') > $(b).attr('sortvalue')) return 1;
				return 0;
			})
		);
	};

	this.loadSearchList = function() {

		//var indexObject = new Array(); //{searches: []}
		//if (window.console) {console.log("loadSearchList() start");}

		var search_list = '';

		//search_list = search_list + '<button id="epgwiz-import-se-button" class="jk-but iconset ic-open" title="' + self.i18.tooltip.se_import + '">' + self.i18.show_se + '</button>';
		//search_list = search_list + '<input type="checkbox" name="se-import" id="epgwiz-se-import" style="display: none;">';
		search_list = search_list + '<label for="epgwiz-se-import" class="epgwiz-se-label" title="' + self.i18.tooltip.se_import + '">' + self.i18.show_se + '</label>';

		search_list = search_list + '<ul id="epgwiz-search-list" class="epgwiz-saved-search-items">';

		$.getJSON(indexFileName, function(savedSearchList){

			$.each(savedSearchList, function(i, searchObject){

				//if (window.console) {console.log("loadSearchList() searchName: '" + searchObject.searchName + "' fileName: '" + searchObject.fileName + "'");}
				//search_list = search_list + '<li sortvalue="' + searchObject.searchName + '" class="epgwiz-search-item epgwiz-saved-search-item" value="' + searchObject.searchName + '" filename="' + searchObject.fileName + '"><table><tr><td style="width: 93%">' + searchObject.searchName + '</td><td><a class="iconset ic-delete epgwiz-saved-search-delete"></a></td></tr></table></li>';

				search_list = search_list + '<li sortvalue="' + searchObject.searchName.toUpperCase() + '" class="epgwiz-search-item epgwiz-saved-search-item" value="' + searchObject.searchName + '" filename="' + searchObject.fileName + '"><i class="iconset ic-delete epgwiz-saved-search-delete"></i><span>' + searchObject.searchName + '</span></li>';

				//search_list = search_list + '<li class="epgwiz-saved-search-item" value="' + searchObject.searchName + '" filename="' + searchObject.fileName + '"> <span>' + searchObject.searchName + '</span></li>';
			});

			search_list = search_list + '</ul>';

			var infoWindow = new jkqWindow();
			infoWindow.width = 320;

			infoWindow.showModal();
			infoWindow.setTitle( self.i18.module.search_title );
			infoWindow.closeOnOutsideClick();

			infoWindow.container.find('DIV.jkqWindowContent')
			.append(
					'<div class="epgwiz-saved-search-list">' +
					search_list +
					'</div>'
			);

			self.sortSearchList();

			//$('.epgwiz-saved-search-item').click( function(e) {self.loadSearchFromList($(this).parents('TR'));});
			//$('.epgwiz-saved-search-item').click( function(e) {self.loadSearchFromList($(this));});


			$('#epgwiz-search-list').on('click', '.epgwiz-saved-search-item', function() {
			//$('.epgwiz-saved-search-item').click( function(e) {
				self.loadSearchDefinition($(this).attr('value'));
				infoWindow.close();
			});

			$('#epgwiz-search-list').on('click', '.epgwiz-saved-search-delete', function() {
			//$('.epgwiz-saved-search-delete').click( function(e) {

				//alert("Deleting " + $(this).parents('LI').attr('value') + " " + $(this).parents('LI').attr('filename'));

				var s_name = $(this).parents('LI').attr('value');
				var s_file = $(this).parents('LI').attr('filename');

				new jkqDialog({
					title: self.i18.delete_hdr,
					text: self.i18.delete_def + " '" + s_name + "'",
					buttons: {
						ok:				 tma.i18.respond_yes,
						jkqDialogCancel: tma.i18.respond_cancel
					}
				}).confirm(function(button) {

					if (button == 'ok') {
						self.deleteSearchDefinition(s_file, s_name);
						$(this).parents('LI').remove();
						self.resetAllCriteria();
					}
				});
			});//END Click

			$('.epgwiz-se-label').click( function(e) {

				if($(this).hasClass('epgwiz-se-label-selected'))
				{
					$('.epgwiz-se-label').removeClass('epgwiz-se-label-selected');

					$('.epgwiz-saved-search-items').find('.epgwiz-se-search-item').remove();

					self.sortSearchList();
				}
				else
				{
					$('.epgwiz-se-label').addClass('epgwiz-se-label-selected');

					//alert("Importing SE Search List");
					//self.loadSearchDefinition($(this).attr('value'));
					//infoWindow.close();

					var nameSaved = "";
					var se_list = "";
					//se_list = se_list + '<ul>';
					var se_count = 0;

					$.get(smartepgSearch, function(seDefinition){
						//alert(seDefinition);
						$.each(seDefinition.split('\n'), function(index, value) {
							//alert(index + ': ' + value);
							//if (window.console) {console.log(index + ': ' + value);}

							//if (window.console) {console.log(value.slice(0,5));}
							if(value.slice(0,5) == "Name:"){nameSaved = value.slice(18);}

							if(value.slice(0,3) == "ID:")
							{
								//nameSaved = value.slice(18); console.log("* " + nameSaved);
								//if (window.console) {console.log(index + ': ' + value);}
								se_list = se_list + '<li sortvalue="' + nameSaved + '" class="epgwiz-search-item epgwiz-se-search-item" value="' + value.slice(18) + '" sename="' + nameSaved + '">[' + nameSaved + ']</li>';
								se_count++;
							}
						});

						//se_list = se_list + '</ul>';

						//if (window.console) {console.log("Found " + se_count.toString() + "SE items.");}

						if(se_count > 0)
						{
							//if (window.console) {console.log("Adding " + se_count.toString() + "SE items.");}
							infoWindow.container.find('UL.epgwiz-saved-search-items')
							.append(
									//'<div class="epgwiz-saved-search-items">' +
									se_list //+
									//'</div>'
							);

							self.sortSearchList();

							$('.epgwiz-se-search-item').click( function(e) {

								//if (window.console) {console.log("Importing SE search item " + $(this).attr('value') + " - " + $(this).attr('sename'));}

								//self.loadSearchDefinition($(this).attr('value'));
								self.loadSEDefinition($(this).attr('sename'), $(this).attr('value'));
								infoWindow.close();

							});

							infoWindow.adjustPos();
						}
					}).fail(function(xhr, errorType, error) {
						if(xhr.status == 404)
						{
							alert(self.i18.se_not_found);  //This should be a rare error so an 'alert' is OK.
						}
					});

				}

			});

			//$('#epgwiz-import-se-button').click( function(e) {
			//
			//
			//});

			infoWindow.adjustPos();

		}).done(function() {
			//if (window.console) {console.log("loadSearchList() done");}
		}).fail(function(xhr, errorType, error) {
			//if (window.console) {console.log("loadSearchList() fail");}
			if(xhr.status == 404)
			{
				alert(self.i18.no_saved_searches);  //This should be a rare error so an 'alert' is OK.
			}
			else
			{
				tma.log({
					title: errorType + " in epgsearch.loadSearchList()",
					details: {
						error: error,
						xhrReadyState: xhr.readyState,
						xhrStatus: xhr.status,
					}
				});
				return 0;
			}
		});

	};//END loadSearchList()


	this.loadSEDefinition = function(se_name, se_id) {

		//if (window.console) {console.log("Importing SE search item " + se_name + " - " + se_id);}

		var searchDefinition = {
			name:		'SmartEPG_' + se_name,
			minlen:		'00:00',
			maxlen:		'00:00',
			startdate:	'',
			enddate:	'',
			starttime:	'00:00',
			endtime:	'00:00',

			favgroup:	'',

			weekday:	{1: false, 2: false, 3: false, 4: false, 5: false, 6: false, 7: false},

			texts:		[], //new Array(),
			//textcount:	0,

			channels:	[], //new Array(),
			//channelcount:0,
		};

		$.get(smartepgSearch, function(seDefinition){
			$.each(seDefinition.split('--------------------------------------------'), function(index, seDef) {

				if((seDef.search("Name:             " + se_name) != -1) && (seDef.search("ID:               " + se_id) != -1))
				{
					$.each(seDef.split('\n'), function(index, seLine) {

						var seKey = seLine.slice(0,17).trim();
						var seValue = seLine.slice(18).trim();
						var tLen = 0;

						switch(seKey){

							case "ChannelID:":
								searchDefinition.channels.push({logoid:	seValue});
							break;

							case "StartDate:":
								tLen = parseInt(seValue, 16);
								if(tLen > 0){searchDefinition.startdate = self.MJDtoJSON(tLen);}
							break;

							case "EndDate:":
								tLen = parseInt(seValue, 16);
								if(tLen > 0){searchDefinition.enddate = self.MJDtoJSON(tLen);}
							break;

							case "StartTime:":
								tLen = parseInt(seValue);
								if(tLen == 6144){tLen = 0;}
								if(tLen > 0){searchDefinition.starttime = ("00" + ((tLen & 0xff00) >> 8).toString()).slice(-2) + ":" + ("00" + (tLen & 0x00ff).toString()).slice(-2);}
							break;

							case "EndTime:":
								tLen = parseInt(seValue);
								if(tLen == 6144){tLen = 0;}
								if(tLen > 0){searchDefinition.endtime = ("00" + ((tLen & 0xff00) >> 8).toString()).slice(-2) + ":" + ("00" + (tLen & 0x00ff).toString()).slice(-2);}
							break;

							case "MinLength:":
								tLen = parseInt(seValue);
								if(tLen > 0){searchDefinition.minlen = ("00" + (Math.floor(tLen / 60)).toString()).slice(-2) + ":" + ("00" + (tLen % 60).toString()).slice(-2);}
							break;

							case "MaxLength:":
								tLen = parseInt(seValue);
								if(tLen > 0){searchDefinition.maxlen = ("00" + (Math.floor(tLen / 60)).toString()).slice(-2) + ":" + ("00" + (tLen % 60).toString()).slice(-2);}
							break;

							case "WeekDays:":

								$(seValue.split('')).each(function (index, character) {
									searchDefinition.weekday[parseInt(character)] = true;
								});

							break;

							case "Search:":

								var searchFields = seValue.split(';');

								var tT = false;
								var tD = false;
								var tE = false;

								switch(searchFields[0])
								{
									case "B": tT = true;  tD = true;  tE = true; 	break;
									case "T": tT = true;  tD = false; tE = false; 	break;
									case "D": tT = false; tD = true;  tE = false; 	break;
									case "A": tT = true;  tD = true;  tE = true; 	break;
									case "E": tT = false; tD = false; tE = true; 	break;
								}

								var tX = searchFields[1].trim();
								tX = tX.replace(/^\*/,'');	//Trim leading * if one exists
								tX = tX.replace(/\*$/,'');	//trim trailing * if one exists

								searchDefinition.texts.push({
									text:	tX,
									title:	tT,
									desc:	tD,
									ext:	tE,
									points:	(Math.round(parseInt(searchFields[2]) / 10) * 10).toString(),
								});
							break;


						}
					});

					//alert(JSON.stringify(searchDefinition));
					self.loadSearchFields(searchDefinition);
				}
			});
		});
	};//END loadSEDefinition()
}

// jQuery epgwiz weekday plugin
$.fn.epgwizWeekday = function() {

	if (arguments.length) {

		// set values
		for (var i = 0; i < arguments.length; i++) {

			var on = arguments[i];
			if (on) {
				this.eq(i).addClass('epgwiz-weekday-selected');
			} else {
				this.eq(i).removeClass('epgwiz-weekday-selected');
			}
			$( '#' + this.eq(i).attr('for') ).prop('checked', on);

		}

	} else {

		// init element
	    this.each(function() {

	    	// get corresponding checkbox
			var checkbox = $( '#' + $(this).attr('for') );
			// hide checkbox
			checkbox.css('display', 'none');
			// init selection state
			if (checkbox.is(':checked')) {
				$(this).addClass('epgwiz-weekday-selected');
			}

	    })
		// add on/off function
	    .click(function(e) {
	    	e.preventDefault();
	    	$(this).toggleClass('epgwiz-weekday-selected');
	    	// find checkbox
	    	$( '#' + $(this).attr('for') )
	    	// and set value
	    	.prop( 'checked', $(this).hasClass('epgwiz-weekday-selected') );
	    });

	}

	return this; // support chaining

};



			// new jkqDialog({
				// title: tma.i18.timer.delete_confirm_caption,
				// text:  tma.i18.timer.delete_confirm_text.format(timer.getFilename(true)),
				// autoSkipText: tma.i18.respond_auto_skip,
				// autoSkipButton: 'tmaDeleteTimer',
				// buttons: {
					// tmaDeleteTimer:	 tma.i18.respond_yes,
					// jkqDialogCancel: tma.i18.respond_cancel
				// }
			// }).confirm(function(button) {

				// if (button == 'tmaDeleteTimer') {

					// tma.apiDeleteTimer(timer).fail(function(xhr, errorType, error) {

						// new jkqAlert({
							// title:	tma.i18.error.dialog_title,
							// text:	tma.i18.error.timer.del + '<br><br>' + error
						// }).confirm();

					// });

				// }

			// });