/**
 * Genre Colours utility module
 */
tma.registerModule( genrecolours );
genrecolours.prototype = new tmaModule();
function genrecolours() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'genrecolours';

	var self = this;

	/**
	 * create a table-columns config element
	 *
	 * @param {Boolean}		initialEnable		is colouring initially enabled?
	 * @param {Function}	onEnableChange		callback function(isEnabled): if enabled value has changed
	 * @returns {jQueryObject} div.setting
	 */
	this.createConfig = function(initialEnable, onEnableChange) {

		var $colSetting = $('<div class="setting" />');

		this.init().done(function() {

			$colSetting.append(
					self.templates.content.render(
							// data
							{
								i18: self.i18.config,
								subGenres: tma.genres.getMainGenres()
							},
							// partials
							{
								genres: self.templates.genres
							}
					)
			);

			// setup module colouring enabler

			$colSetting.find('INPUT#module-genre-color-enable').click( function() {
				// save setting
				if (onEnableChange) onEnableChange($(this).is(':checked'));
			})
			// init
			.prop('checked', initialEnable);

			// setup color enabler
			$colSetting.on('click', 'INPUT.genre-color-enable', function() {
				var genre = tma.genres.genre( parseInt($(this).parents('LI').attr('genre-index')) );
				genre.coloring( $(this).is(':checked') );
				$(this).siblings('.genre-color-name').attr('style', genre.styleString() );
				$(this).siblings('INPUT.genre-color-text').spectrum( genre.coloring() ? 'enable' : 'disable' );
				$(this).siblings('INPUT.genre-color-bg'  ).spectrum( genre.coloring() ? 'enable' : 'disable' );
				$(tma).trigger('genre_colors_change', [genre]);
			});

			var spectrumInitialised = false;
			// collapsible
			$colSetting.on('click', '.jk-collapser', function() {

				if (!spectrumInitialised) {
					// initialise spectrum color pickers only, if genre-config is expanded
					$colSetting.find('INPUT.genre-color-text').spectrum({
						allowEmpty: true,
						showInitial: true,
						preferredFormat: "hex",
						showInput: true,
						replacerClassName: 'sp-replacer-text-color',
						change: function(color) {
							var col = (color) ? color.toHexString() : '';
							$(this).siblings('.genre-color-name').css('color', col);
							// set genre property
							var genre = tma.genres.genre( parseInt($(this).parents('LI').attr('genre-index')) );
							genre.text_color(col);
							$(tma).trigger('genre_colors_change', [genre]);
						},
					});

					$colSetting.find('INPUT.genre-color-bg').spectrum({
						allowEmpty: true,
						showInitial: true,
						preferredFormat: "hex",
						showInput: true,
						change: function(color) {
							var col = (color) ? color.toHexString() : '';
							$(this).siblings('.genre-color-name').css('backgroundColor', col);
							// set genre property
							var genre = tma.genres.genre( parseInt($(this).parents('LI').attr('genre-index')) );
							genre.background(col);
							$(tma).trigger('genre_colors_change', [genre]);
						},
					});

					spectrumInitialised = true;
				}

				var $icon = $(this).find('I');
				var $collapsible = $(this).siblings('.jk-collapsable');
				$collapsible.slideToggle(function() {
					$collapsible.toggleClass('jk-expanded jk-collapsed');
					$icon.toggleClass('fa-chevron-right fa-chevron-down');
				});

			});

			// drag&drop colours
			$colSetting.on('mousedown', 'DIV.sp-preview', function(e) {

				self.dragndrop.start(e, this, 500);

			});

		});

		return $colSetting;

	};

	/**
	 * Drag&Drop Objekt
	 */
	this.dragndrop = new jkqDragDrop(
			// Drop-Targets
			'DIV.sp-replacer:visible',
			// onDragDrop
			function(target, source) {

				var spectrumCol = $(source).parent().siblings('INPUT').spectrum("get");
				var col = (spectrumCol && spectrumCol.toHexString) ? spectrumCol.toHexString() : '';

				var $tgtInput = $(target).siblings('INPUT');

				$tgtInput.spectrum('set', spectrumCol );

				// set genre property
				var genre = tma.genres.genre( parseInt($(target).parents('LI').attr('genre-index')) );
				if ($tgtInput.is('.genre-color-text')) {
					genre.text_color(col);
				} else if ($tgtInput.is('.genre-color-bg')) {
					genre.background(col);
				}

				$(tma).trigger('genre_colors_change', [genre]);

			}//,

			// onDragOver: default behaviour

			// onDragOut:  default behaviour

	);

	/** @param {TmaGenre} genre */
	$(tma).on('genre_colors_change', function(e, genre) {
		// genre color configuration has changed

		// update the list item
		var $li = $('.setting LI[genre-index="' + genre.genre_value + '"]');

		$li.find('> INPUT.genre-color-enable').prop('checked', genre.coloring() );
		$li.find('> INPUT.genre-color-text')
			.spectrum('set', genre.text_color() )
			.spectrum( genre.coloring() ? 'enable' : 'disable' );
		$li.find('> INPUT.genre-color-bg'  )
			.spectrum('set', genre.background() )
			.spectrum( genre.coloring() ? 'enable' : 'disable' );
		$li.find('> .genre-color-name').attr('style', genre.styleString() );

		// refresh sub-genres
		$.each(genre.subGenres(), function(i, subGenre) {
			var $li = $('.setting LI[genre-index="' + subGenre.genre_value + '"]');
			$li.find('> .genre-color-name').attr('style', subGenre.styleString() );
		});

	});

}
