/**
 * multi channel select utility module
 */
tma.registerModule( multichselect );
multichselect.prototype = new tmaModule();
function multichselect() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'multichselect';

	var self = this;

	/**
	 * init and show selector
	 *
	 * @param {jQueryObject}	$select		associated select field
	 * @param {Object}			options		optional
	 *  options:
	 * 		enableFavSelect:	{Boolean} 	default true,
	 *		onFavChange:		{Function(favGroup)}	default $.noop,
	 */
	this.showSelector = function($select, options) {

		var defaults = {
			enableFavSelect:	true,
			selectedFavGroup:	tma.localSetting('defaultfavourites', ''),	//The calling function can specify a starting favourites group or take the default group.
			onFavChange:		$.noop,	// equivalent to: function() {}
		};

		this.init().done(function() {

			self.createPopup($select, $.extend({}, defaults, options));

		});
	};

	/**
	 * create window
	 *
	 * @param {jQueryObject}	$select		associated select field
	 * @param {Object}			options		optional options
	 * @returns {jkqWindow}
	 */
	this.createPopup = function($select, options) {

		var popup = new jkqWindow();
		popup.width = 350;

		popup.show();
		popup.setTitle( self.i18.heading );
		popup.closeOnOutsideClick();

		var $content = popup.container.find('DIV.jkqWindowContent');

		// TODO: add template files
		var template =
			'<div class="multichselector">' +
				'<div class="chs-fav">' +
					'<select name="favgroup">' +
						'<option value="">{all}</option>' +
						'<option value="__tv">{tv}</option>' +
						'<option value="__radio">{radio}</option>' +
					'</select>' +
				'</div>' +
				'<ul />' +
			'</div>';

		var item_tmpl =
			'<li channelid="{logoid}">' +
				'<i class="channellogo logo-{logoid} haslogo-{haslogo}" />' +
				'<span>{servicename}</span>' +
			'</li>';

		var $selector = $( template.format(tma.i18.favgroups) )
		.appendTo($content);

		if (!options.enableFavSelect) {
			$selector.find('.chs-fav').hide();
		}

		// fill fav select
		var $favselect = $selector.find('SELECT[name="favgroup"]');
		/** @param {TmaFavouritesGroup} group */
		tma.favourites.eachGroup( function(nr, group) {

			if (group.count > 0) {
				//Only list groups with members
				$favselect.append(
						('<option value="{name}">{getName}</option>').format(group)
				);
			}
		});

		//var favGroup = tma.localSetting('defaultfavourites', '');	//DMC - Take the favourites group passed from the calling function.
		var favGroup = options.selectedFavGroup;
		var favChannels = tma.getChannels(favGroup);
		var selectedid = '';

		var $ul = $selector.find('UL');

		$favselect.change(function() {

			favGroup = $(this).val();
			favChannels = tma.getChannels(favGroup);

			options.onFavChange.call($favselect, favGroup);

			$ul.empty();

			/** @param {TmaChannel} channel */
			$.each(favChannels, function(ix, channel) {

				var $option = $select.find('OPTION[value="' + channel.logoid + '"]');
				if ( $option.length ) {
					// channel exists in select field
					var $li = $(item_tmpl.format(channel)).appendTo($ul);
					if ( $option.prop('selected') ) {
						// adapt selection
						$li.addClass('selected');
					}
				}
			});

			// call optional onFavChange callback with $favselect as 'this' argument
			//options.onFavChange.call($favselect, favGroup);

		})
		// initialise
		.val(favGroup).change();

		$ul.on('click', 'LI', function() {

			$(this).toggleClass('selected');
			var channelid = $(this).attr('channelid');
			$select.find('OPTION[value="' + channelid + '"]').prop('selected', $(this).hasClass('selected'));
			$select.change(); //DMC - Generate a change event so that the calling module can react if necessary.
		});

		popup.adjustPos({
			hAlign:	'center',
			effect:	'slide-rtl'
		});

		return popup;

	};

}
