/**
 * recordings module
 */
tma.registerModule( recordingspage );
recordingspage.prototype = new tmaModule();
function recordingspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recordingspage';

	this.pageGroup = 'files';

	// available table columns
	var tableColumns = {
		filename:	true,
		filefn:		true,
		filesize:	true,
		filedate:	true,
		duration:	true,
		channel:	true
	};

	var self = this;

	/**
	 * js timer, checks status periodically (set by this.playFile)
	 */
	this.statusInterval = null;

	/**
	 * setup module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );
		tma.fn.addFeature( this, 'playFile' );
		tma.fn.addFeature( this, 'startTap' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * display the content
	 *
	 * @returns jQueryDeferred
	 */
	this.show = function() {

		return this.init().done(function() {

			if (tma.files === null) {
				// load the files
				var dir = $('DIV.directory A[href]').last().attr('href') || "/DataFiles";
				self.loadFiles(dir);
			} else {
				// show existing files
				self.showFiles(tma.files);
			}

			tma.call('addConfigItem', ['MainMenu', self]);

			self.insertConfig();

		});

	};

	/**
	 * reload data and display the content
	 */
	this.refresh = function() {

		// refresh the files
		var dir = $('DIV.directory A[href]').last().attr('href') || "/DataFiles";
		self.loadFiles(dir);

	};

	/**
	 * this page gets removed
	 */
	this.cleanup = function() {

		// stop status checking
		if (self.statusInterval) {
			window.clearTimeout(self.statusInterval);
			self.statusInterval = null;
		}

		// hide and clear config frame, if exists
		$('DIV#config').removeClass('recordings-config');

	};

	/**
	 * get a new directory content
	 *
	 * @returns jQueryDeferred
	 */
	this.loadFiles = function (directory) {

		// delete all child nodes of content and display a pending image
		$('#content').append('<img src="css/img/wait.gif">');
		$("#timestamp").text(this.i18.files_loading);

		return tma.apiGetFiles(directory).always(function() {

			// remove pending image
			$('#content').empty();

		}).done(function(files) {

			self.showFiles(files);

		}).fail(function(xhr, errorType, error) {

			$("#timestamp").append( tma.i18.error.ajax.format(errorType, error) );

		});

	};

	/**
	 * display path breadcrumbs
	 */
	this.renderBreadcrumbs = function() {

		var path = tma.files.directory.replace(/^\//, '').split('/');
		var href = '';
		var mount_depth = 1;
		var bpath = [];
		var selected = '';
		$.each(path, function(nr, value) {

			href += '/' + value;

			if ( (nr == 1) && (value == 'hd')) {
				mount_depth = 2;
			}

			var item = {
				href: '',
				name: value,
			};

			if ( nr >= mount_depth ) {

				item.href = href;

				if ( nr == mount_depth ) {
					selected = href;
				}

			}

			bpath.push(item);

		});

		$( this.templates.breadcrumbs.render({
			path: bpath
		})).appendTo('DIV#content')

		.on('click', 'A', function(e) {
			e.preventDefault();
			self.loadFiles( $(this).attr('href') );
		})

		.on('change', 'SELECT', function(e) {
			self.loadFiles( $(this).val() );
		})

		// set selected option
		.find('SELECT').val(selected);

	};

	/**
	 * display the file list of the current files object
	 *
	 * @param {TmaFiles}	files
	 */
	this.showFiles = function (files) {

		// delete all child nodes of content
		$('#content').empty();

		$("#timestamp"	).text( new Date(tma.files.timestamp).toLocaleString() );

		$('DIV#content').append( this.templates.diskspace.render({
			disk: this.getDiskSpace(),
			i18: this.i18,
		}));

		this.setFilesStatus();

		if (!tma.files) return;

		var channelDisplay = parseInt( this.localSetting('channel_display', 3) );
		var userCols = this.localSetting( 'columns', tableColumns );
		var inProgramFiles = tma.files.directory.indexOf('/mnt/hd/ProgramFiles') === 0;

		// sort files
		var sortKey = this.localSetting('sort_key', 'name');
		var sortDir = this.localSetting('sort_dir', 'asc');
		tma.sortFiles(sortKey, sortDir);

		// display path breadcrumbs
		this.renderBreadcrumbs();

		var $table = $( this.templates.filelist.render({
			i18: this.i18,
			files: files.files,
			formatFilename: function() {
				if ( this.tappresent ) {
					return this.tap.name;
				} else {
					var removeExt = ( this.infpresent || this.tappresent );
					return this.getName( removeExt );
				}
			},
			formatFilesize: function() {
				if (this.attr == 208) {
					if (		this.size /       1024 /*1KB*/ < 0.1 ) {
						return ( this.size ).toLocaleString() + ' B';
					} else if (	this.size /    1048576 /*1MB*/ < 0.1 ) {
						return (parseInt(this.size * 10 /       1024 /*1KB*/) / 10 ).toLocaleString() + ' KB';
					} else if (	this.size / 1073741824 /*1GB*/ < 0.1 ) {
						return (parseInt(this.size * 10 /    1048576 /*1MB*/) / 10 ).toLocaleString() + ' MB';
					} else {
						return (parseInt(this.size * 10 / 1073741824 /*1GB*/) / 10 ).toLocaleString() + ' GB';
					}
				} else {
					return '';
				}
			},
			formatFiledate: function() {
				if ( this.isFile() ) {
					if ( (this.infpresent) && (this.infdata.header.starttime)) {
						return tma.date( self.i18.recdate_fmt, this.infdata.header.starttime );
					} else {
						return tma.date( self.i18.recdate_fmt, this.timestamp );
					}
				} else {
					return '';
				}
			},
			formatDuration: function() {
				if (this.infdata) {
					return tma.formatDuration(this.infdata.header.durationmin);
				} else if (this.tap) {
					return this.tap.version;
				} else {
					return '';
				}
			},
			channel_mode: channelDisplay,
		}) ).appendTo('#content');

		// sort table columns in user defined order
		$('#content TABLE.files TR').each(function() {
			$(this).find('TD,TH').sort(function(a,b) {
				return Object.keys(userCols).indexOf($(a).attr('class')) - Object.keys(userCols).indexOf($(b).attr('class'));
			}).appendTo(this);
		});

		// hide deselected columns
		$.each(userCols, function(clmClass, display) {
			if (
					!display ||
					(
							// also hide channel column, if not in DataFiles
							(clmClass == 'channel') &&
							( tma.files.directory.indexOf('/mnt/hd/DataFiles') !== 0 )
					)
			) {
				$table.find( ( 'TD.{0},TH.{0}' ).format( clmClass ) ).hide();
			}
		});

		// if ProgrammFiles, change duration caption
		if ( inProgramFiles ) {
			$table.find('TH.duration A').text('Version');
		}

		// set the current sort state
		$table.find('TH[sort-key="' + sortKey + '"]').attr({
			'sort-dir':		sortDir,
			'can-sort-dir': (sortDir == 'asc') ? 'desc' : 'asc'
		});

		// set sort event handler
		$table.find('THEAD').on('click', 'A', function(e) {
			e.preventDefault();
			var sDir = $(this).parent().attr('can-sort-dir');
			var sKey = $(this).parent().attr('sort-key');
			self.setLocalSetting('sort_key', sKey);
			self.setLocalSetting('sort_dir', sDir);
			self.showFiles(files);
		});

		// attach file objects to rows
		$('#content TR[inode]').each(function() {
			var f = files.find( $(this).attr('inode') );
			if (f) {
				$(this).prop('file', f);
			}
		});

		// add event listener for folders to change directories
		$table.find('TR[attr="224"]').on('click', function() {

			// 'this' is the clicked element
			var f = $(this).prop('file');
			//var f = files.find( $(this).attr('inode') );
			if (f) {
				var directory = tma.files.directory + '/' + f.name;
				self.loadFiles( directory );
			}
		});

		// add event listener to show infos
		$table.on('click', 'BUTTON.info', function(e) {

			var $row = $(this).parents('TR');

			if (tma.debug) tma.log('recordingspage.info.click');

			// check, if any module handles the 'showRecInf' event
			// and prevent reopening of the same info
			if (
					(tma.fn.showRecInf) &&
					!$row.hasClass('jk-mark')
			) {

				if (tma.debug) tma.log('recordingspage.info.click: showRecInf available');

				var f = $row.prop('file');

				if (tma.debug) tma.log('recordingspage.info.click: f=' + f.name);

				tma.fn.showRecInf( f ).done(function(popup) {

					popup.file = f;
					$(popup).on('close', function() {
						$('TABLE.files TR[inode="' + this.file.inode + '"]').removeClass('jk-mark');
					});

				});

				$row.addClass('jk-mark');

			}

		});

		// add event listener to show "more functions"
		$table.on('click', 'BUTTON.morefn', function() {

			// remove previous moreFn's
			$('DIV.morefn').fadeOut(function() {
				$(this).remove();
			});

			$(this).parents('TR').addClass('jk-mark');

			var $moreFn = $( self.templates.morefn.render({
				i18: self.i18
			}) );

			/** @returns {TmaFile} */
			var file = $(this).parents('TR').prop('file');

			tma.call('fileCreateMoreFn', [$moreFn, file]);

			$moreFn.appendTo( $(this).parent() ).fadeIn();

		});

		// add event listener to play files
		$table.on('click', 'BUTTON.play', function() {

			var $tr = $(this).parents('TR');
			/** @returns {TmaFile} */
			var f = $tr.prop('file');

			// start playback
			if ( $tr.is('.playable') ) {

				var startposition = 0;
				if (f.infpresent) {
					startposition = f.infdata.bookmarks.startposition || 0;
				}

				self.playFile(null, tma.files.directory, f, startposition);

			}

		});

		// add event listener to start TAPs
		$table.on('click', 'BUTTON.start', function() {

			var $tr = $(this).parents('TR');
			/** @returns {TmaFile} */
			var f = $tr.prop('file');

			// start TAP
			if ( $tr.is('.startable') ) {

				self.startTap(null, tma.files.directory, f);

			}

		});

		// add event listener to stop playback or tap
		$table.on('click', 'BUTTON.stop', function() {

			var $tr = $(this).parents('TR');

			if ($tr.is('.play')) {
				tma.stopPlayback().done(function() {

					$tr.removeClass('play');

					if (self.statusInterval) {
						window.clearTimeout(self.statusInterval);
						self.statusInterval = null;
					}

				});
			}

			if ($tr.is('.running')) {

				/** @returns {TmaFile} */
				var f = $tr.prop('file');
				// find running TAP
				var tap = tma.taps.find(f.tap.tapid, tma.files.directory + '/' + f.name);
				if (tap) {
					tma.apiStopTap(tap).done(function() {
						$tr.removeClass('running');
					});
				} else {
					$tr.removeClass('running');
				}
			}

		});

		// add event listener to stop recording
		$table.on('click', 'BUTTON.stoprec', function() {

			var $tr = $(this).parents('TR');
			var slot = parseInt( $tr.attr('slot') );
			tma.stopRecording(slot).done(function(data) {

				if (data.return_code == 1) {
					$tr.removeClass('rec');
				}

			});

		});

		// add event listener to delete files
		$table.on('click', 'BUTTON.ic-delete', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			self.deleteFile(null, tma.files.directory, f);
		});

		// add event listener to rename files
		$table.on('click', 'BUTTON.ic-rename', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			self.renameFile(null, tma.files.directory, f);
		});

		// add event listener to move files
		$table.on('click', 'BUTTON.ic-move', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			// open a directory selector...
			if (tma.featureExist('selectDirectory')) {
				tma.fn.selectDirectory( '/mnt', tma.files.directory, function(selectedPath) {
					self.moveFile(null, tma.files.directory, f, selectedPath);
				});
			}

		});

		// add event listener to open files
		$table.on('click', 'BUTTON.open', function() {

			/** @returns {TmaFile} */
			var f = $(this).parents('TR').prop('file');

			// open a text file in a new window
			window.open(tma.files.directory + '/' + f.name,'_blank');

		});

		// close "more functions"
		$table.on('click', 'DIV.morefn BUTTON', function() {
			$(this).parent().fadeOut(function() {
				$(this).remove();
			});
			$(this).parents('TR').removeClass('jk-mark');
		});

		$('DIV#content').append($table);

	};

	this.getDiskSpace = function() {

		var freeGB = 0;
		var totalGB = 0;

		if (tma.status) {
			freeGB = tma.status.freedisk >> 10 || 0;
			totalGB = tma.status.totaldisk >> 10 || 0;
		}
		var usedGB = totalGB - freeGB;

		return {
			usedGB: usedGB,
			freeGB: freeGB,
			totalGB: totalGB,
			usedPercent: parseInt(usedGB * 1000 / totalGB) / 10,
			freePercent: parseInt(freeGB * 1000 / totalGB) / 10,
		};

	};

	/**
	 * sets sizes and displayed values (used/free/total) of the current drive
	 */
	this.setDiskSpace = function() {

		var disk = this.getDiskSpace();

		$('.diskspace-used')
		.css('width', disk.usedPercent + '%')
		.find('.diskspace-value').text(disk.usedGB);

		$('.diskspace-free')
		.css('width', disk.freePercent + '%')
		.find('.diskspace-value').text(disk.freeGB);

		// set total space
		$('.diskspace-total')
		.find('.diskspace-value').text(disk.totalGB);

		// set free space in percent
		$('.diskspace-free-percent')
		.find('.diskspace-value').text(disk.freePercent);

	};

	/**
	 * loads the status via API and updates diskspace and file's play/rec state
	 */
	this.setFilesStatus = function () {

		/** @param {TmaStatus} status */
		tma.apiGetStatus().done(function(status) {

			self.setDiskSpace();

			$('TABLE.files TBODY TR.playable').removeClass('rec play');

			if (self.statusInterval) {
				window.clearTimeout(self.statusInterval);
				self.statusInterval = null;
			}

			if (status.absolutefilename) {
				// file is playing
				var file = tma.splitPath(status.absolutefilename);
				if (file) {
					if (tma.files.directory == file.directory) {
						$('TABLE.files TR').filter(function() {
							/** @returns {TmaFile} */
							var f = $(this).prop('file');
							return (f) &&
							(
									(f.getName() + '.inf' == status.getFilename()) ||
									(f.getName() == status.getFilename())
							);
						}).addClass('play');
					}
				}

			}

			if (tma.files.directory == '/mnt/hd/DataFiles') {

				status.eachRecording( function(nr, recording) {

					$('TABLE.files TR').filter(function() {
						var f = $(this).prop('file');
						return (f) && (f.name_hex == recording.filename_hex);
					}).addClass('rec').attr('slot', recording.slot);

				});

			}

			// remove external mount points
			$('DIV.directory SELECT OPTION[external]').remove();

			if (status.drive_count > 0) {
				var $dirSelect = $('DIV.directory SELECT');
				$.each(status.drives, function(nr, drive) {

					$('<option value="' + drive.mountpath + '" external="true">' + drive.mountpath + '</option>').appendTo($dirSelect);

					if ( tma.files.directory.indexOf(drive.mountpath) === 0 ) {
						// this is the current drive
						$dirSelect.val(drive.mountpath);

						self.setDiskSpace();
					}

				});
			}

			// Tap status
			if ( tma.files.directory.indexOf('/mnt/hd/ProgramFiles') === 0 ) {
				tma.apiGetTaps().done(function() {

					$('TABLE.files TBODY TR.startable').removeClass('running');

					/** @param {TmaTap} tap */
					tma.taps.eachTap(function(i, tap) {

						if ( tap.file_name.replace(/\/[^/]+$/, '') == tma.files.directory ) {
							var $row = $('TABLE.files TBODY TR[tapid="' + tap.tapid + '"]')
							.addClass('running');

							// update name, author & version
							$row.find('TD.filename SPAN').text( tap.name );
							$row.find('TD.duration SPAN').text( tap.version );
							$row.find('TD.channel  SPAN').text( tap.author );
						}

					});

				});
			}

			// periodically check the status
			self.statusInterval = window.setTimeout(
				function() {
					self.setFilesStatus();
				},
				5000
			);

		});

	};

	/**
	 * displays an alert-box with a file operation error message
	 *
	 * @param {String} fileAction	Name of the file operation
	 * @param {Object} data			API result data
	 */
	this.fileError = function(fileAction, data) {

		var errmsg = '';
		if ( (data.mpg_status !== 0) && (data.mpg_errno) ) {
			errmsg += '<i>{recext}</i>: {mpg_error}<br>';
		}
		if ( (data.inf_status !== 0) && (data.inf_errno) ) {
			errmsg += '<i>inf</i>: {inf_error}<br>';
		}
		if ( (data.nav_status !== 0) && (data.nav_errno) ) {
			errmsg += '<i>nav</i>: {mpg_error}<br>';
		}

		new jkqAlert({
			title: tma.i18.error.dialog_title,
			text: (
					'{message}' +
					'<span style="display: block; text-align: left; margin-top: 0.5em">' +
						errmsg +
					'</span>'
			).format({
				message:	fileAction,
				recext:		tma.config.recextension,
				mpg_error:	(
						tma.i18.error.files.errno[data.mpg_errno] ?
								tma.i18.error.files.errno[data.mpg_errno] :
									'errno = ' + data.mpg_errno
				),
				inf_error:	(
						tma.i18.error.files.errno[data.inf_errno] ?
								tma.i18.error.files.errno[data.inf_errno] :
									'errno = ' + data.nav_errno
				),
				nav_error:	(
						tma.i18.error.files.errno[data.nav_errno] ?
								tma.i18.error.files.errno[data.nav_errno] :
									'errno = ' + data.inf_errno
				),
			})
		}).confirm();

	};

	/**
	 * Feature handler: rename a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @param {String}		newDirectory
	 */
	this.moveFile = function (f, directory, file, newDirectory) {

		if (tma.files.directory != newDirectory) {

			tma.apiRenameFile(
					directory, file.name, file.name_hex,
					newDirectory, file.name
			).done(function(data) {

				if (
						(data.mpg_status === 0) &&
						(data.inf_status === 0) &&
						(data.nav_status === 0)
				) {

					$('TABLE.files TR[inode="' + file.inode + '"]').remove();

				} else {

					self.fileError(tma.i18.error.files.move, data);

				}

			});
		}

	};

	/**
	 * Feature handler: delete a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 */
	this.deleteFile = function(f, directory, file) {

		new jkqDialog({
			title: self.i18.delete_confirm_caption,
			text: self.i18.delete_confirm.format(file.getName(true)),
			buttons: {
				ok:				 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'ok') {

				tma.apiDeleteFile(directory, file.name, file.name_hex).done(function(data) {

					if (
							(data.mpg_status === 0) &&
							(data.inf_status === 0) &&
							(data.nav_status === 0)
					) {

						$('TABLE.files TR[inode="' + file.inode + '"]').remove();

						self.setFilesStatus();

					} else {

						self.fileError(tma.i18.error.files.del, data);

					}

				});

			}

		});

	};

	/**
	 * Feature handler: rename a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 */
	this.renameFile = function(f, directory, file) {

		new jkqInputDialog({
			title: self.i18.rename_dialog_caption,
			text: self.i18.rename_dialog_text.format( file.getName(file.infpresent) ),
			inputValue: file.getName(file.infpresent).replace(/"/, '&quot;')
		}).confirm(function() {

			tma.apiRenameFile(
					directory, file.name, file.name_hex,
					directory, this.value() + ((file.infpresent) ? tma.config.recextension : '')
			).done(function(data) {

				if (
						(data.mpg_status === 0) &&
						(data.inf_status === 0) &&
						(data.nav_status === 0)
				) {

					$('TABLE.files TR[inode="' + file.inode + '"] TD.filename SPAN').text( file.getName( file.infpresent || file.tappresent) );

				} else {

					self.fileError(tma.i18.error.files.rename, data);

				}

			});

		});

	};

	/**
	 * Feature handler: play a file
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file				tma file object
	 * @param {Number}		startposition
	 */
	this.playFile = function (f, directory, file, startposition) {

		tma.apiPlayFile(directory, file.name, file.name_hex, startposition).done(function(playObject) {

			if (playObject.status === 0) {

				$('TABLE.files TR').removeClass('play')
					.filter('TR[inode="' + file.inode + '"]').addClass('play');

				// periodically check the status
				self.statusInterval = window.setTimeout(
					function() {
						self.setFilesStatus();
					},
					2000
				);

			} else {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.files.play
				}).confirm();

			}

		});

	};

	/**
	 * Feature handler: start a TAP
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		directory
	 * @param {TmaFile}		file
	 */
	this.startTap = function (f, directory, file, startposition) {

		tma.apiStartTap(directory + '/' + file.name).done(function(result) {

			if (result.return_code === 1 && result.error === 0) {

				$('TABLE.files TR[tapid="' + file.tap.tapid + '"]').addClass('running');

			} else {

				new jkqAlert({
					title:	tma.i18.error.dialog_title,
					text:	tma.i18.error.files.tap_start
				}).confirm();

			}

		}).fail(function() {

			new jkqAlert({
				title:	tma.i18.error.dialog_title,
				text:	tma.i18.error.files.tap_start
			}).confirm();

		});

	};

	this.insertConfig = function () {

		// create column config
		var $colSetting = createColumnsConfig(
				this.localSetting( 'columns', tableColumns ),
				this.i18.config.label_columns,
				$.extend({filefn:	'Buttons'}, this.i18.columns),
				'#content TABLE.files',
				function(cfg) {
					self.setLocalSetting('columns', cfg );
				}
		);

		// logo/text
		var $channelSetting = createChannelConfig(
				this.localSetting('channel_display', 3),
				this.i18.config.channel_display.caption,
				this.i18.config.channel_display,
				null,
				function(cfg) {
					self.setLocalSetting('channel_display', cfg );
					$('#content .files').attr('channel_mode', cfg);
				}
		);

		$('DIV#config')
		.append($colSetting)
		.append($channelSetting)
		.addClass('recordings-config');


	};

}
