/**
 * timers module
 */
tma.registerModule( timerspage );
timerspage.prototype = new tmaModule();
function timerspage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'timerspage';

	this.pageGroup = 'timers';

	// available table columns
	var tableColumns = {
			timertype:	true,
			filename:	true,
			starttime:	true,
			endtime:	true,
			duration:	true,
			channel:	true,
			frequency:	true,
			timerfn:	true
	};

	var self = this;

	/**
	 * initialise module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the timers, obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#content').html('<img src="css/img/wait.gif">');

			tma.apiGetTimers().done(function() {

				self.showTimers();

				self.insertConfig();

				tma.call('addConfigItem', ['MainMenu', self]);

			}).fail(function(xhr, errorType, error) {

				$("#timestamp").text( tma.i18.error.ajax.format(errorType, error) );
				// remove pending image
				$('#content').empty();

			});

		});

	};

	/**
	 * refresh the timers, obtained from the PVR in the browser window
	 */
	this.refresh = function() {

		$('#content').html('<img src="css/img/wait.gif">');

		tma.apiGetTimers().done(function() {

			self.showTimers();

		}).fail(function(xhr, errorType, error) {

			$("#timestamp").text( tma.i18.error.ajax.format(errorType, error) );
			// remove pending image
			$('#content').empty();

		});

	};

	/**
	 * this page gets removed, so hide the channels config
	 */
	this.cleanup = function() {

		// remove events
		$(tma).off('tmaTimerDeleted.timerspage');
		$(tma).off('tmaTimerAdded.timerspage');
		$(tma).off('tmaTimerModified.timerspage');

	};

	this.showTimers = function () {

		// delete all child nodes of content
		$('#content').empty();

		$("#timestamp"	).text( new Date(tma.timers.timestamp).toLocaleString() );

		// Settings
		var timersFilter = parseInt( this.setting('timersfilter', 0) );
		var channelDisplay = parseInt( this.localSetting('channel_display', 3) );
		var userCols = this.localSetting( 'columns', tableColumns );

		// sort timers
		var sortKey = this.localSetting('sort_key', 'starttime');
		var sortDir = this.localSetting('sort_dir', 'asc');
		tma.sortTimers(sortKey, sortDir);

		$('DIV#content').append( this.templates.timerlist.render({
			timers: tma.timers.timers,
			i18: this.i18,
			filenameNoExt: function() { return this.getFilename(true); },
			formatStarttime: function() {  return tma.date( self.i18.timerdate_fmt, this.starttime ); },
			formatEndtime: function() { return tma.date( "H:i", this.endtime ); },
			formatDuration: function() { return tma.formatDuration(this.duration); },
			formatFrequency: function() { return tma.i18.frequency[this.reservationtype()].name; },
			timersFilter: timersFilter,
			channel_mode: channelDisplay,
		}));

		// adjust paragraph width to table width, to get add button centred
		$('DIV#content P.add-timer').width( $('DIV#content .timers').width() );

		var $table = $('#content TABLE.timers');

		// sort table columns in user defined order
		$('#content TABLE.timers TR').each(function() {
			$(this).find('TD,TH').sort(function(a,b) {
				return Object.keys(userCols).indexOf($(a).attr('class')) - Object.keys(userCols).indexOf($(b).attr('class'));
			}).appendTo(this);
		});

		// hide deselected columns
		$.each(userCols, function(clmClass, display) {
			if (!display) $table.find( ( 'TD.{0},TH.{0}' ).format( clmClass ) ).hide();
		});

		// set the current sort state
		$table.find('TH[sort-key="' + sortKey + '"]').attr({
			'sort-dir':		sortDir,
			'can-sort-dir': (sortDir == 'asc') ? 'desc' : 'asc'
		});

		// set sort event handler
		$table.on('click', 'THEAD A', function(e) {
			e.preventDefault();
			var sDir = $(this).parent().attr('can-sort-dir');
			var sKey = $(this).parent().attr('sort-key');
			self.setLocalSetting('sort_key', sKey);
			self.setLocalSetting('sort_dir', sDir);
			self.showTimers();
		});

		// add event listener to add a timer
		$('DIV#content BUTTON.add-timer').click( function(e) {
			// stop event propagation, needed for 'closeOnOutsideClick'
			e.stopPropagation();

			tma.call('editTimer', [ new TmaTimer({
				index:		-1,		// new timer
				isrec:		1,		// record
			})  ] );
		});

		// add event listener to delete timers
		$('.timers BUTTON.delete-timer').click(function() {

			var timer = tma.timers.find( $(this).parents('TR').attr('timerid') );
			if ( timer ) {
				self.deleteTimer( timer );
			}

		});

		// add event listener to edit timers
		$('.timers BUTTON.edit-timer').click(function(e) {

			var $row = $(this).parents('.timer');
			var timer = tma.timers.find( $row.attr('timerid') );
			if ( timer ) {
				tma.call( 'editTimer', [ timer ] ).done(function(popup) {

					popup.relTimer = timer;
					$(popup).on('close', function() {

						var closedPopup = this;
						// remove current mark
						$('.timers .timer[timerid="' + closedPopup.relTimer.id + '"]')
						.removeClass('jk-mark');

					});

					$row.addClass('jk-mark');

				});
			}

		});

		if (tma.fn.findEvent) {

			// add event listener to view timers event
			$('.timers .timer .filename').click(function(e) {

				var $row = $(this).parents('.timer');
				var timer = tma.timers.find( $row.attr('timerid') );
				if ( timer ) {
					var options = {
						channel: 	tma.channel(timer.logoid),
						eventid:	timer.eventid2,
						starttime:	timer.starttime,
						endtime:	timer.endtime,
					};
					if ( options.channel ) {
						tma.call( 'findEvent', [ options ] ).done(function(popup) {

							popup.relTimerId = timer.id;
							$(popup).on('close', function() {

								var closedPopup = this;
								// remove current mark
								$( '.timers .timer[timerid="' + closedPopup.relTimerId + '"]' ).removeClass('jk-mark');

							});

							$row.addClass('jk-mark');

						});
					}
				}

			});

		}

		// handle modified or added timer
		$(tma).on('tmaTimerModified.timerspage tmaTimerAdded.timerspage', function() {

			// reload timers
			tma.apiGetTimers().done(function() {
				self.showTimers();
			});

		});

		// handle deleted timer
		/** @param {TmaTimer} deletedTimer */
		$(tma).on('tmaTimerDeleted.timerspage', function(e, deletedTimer) {

			// remove reservation
			$( '.timers .timer[timerid="' + deletedTimer.id + '"]' ).remove();

		});

	};

	/**
	 * deletes a reservation
	 *
	 * @param {TmaTimer}	timer
	 */
	this.deleteTimer = function(timer) {

		new jkqDialog({
			title: tma.i18.timer.delete_confirm_caption,
			text:  tma.i18.timer.delete_confirm_text.format( timer.getFilename(true) ),
			autoSkipText: tma.i18.respond_auto_skip,
			autoSkipButton: 'tmaDeleteTimer',
			buttons: {
				tmaDeleteTimer:	 tma.i18.respond_yes,
				jkqDialogCancel: tma.i18.respond_cancel
			}
		}).confirm(function(button) {

			if (button == 'tmaDeleteTimer') {

				tma.apiDeleteTimer(timer).fail(function(xhr, errorType, error) {

					new jkqAlert({
						title:	tma.i18.error.dialog_title,
						text:	tma.i18.error.timer.del + '<br><br>' + error
					}).confirm();

				});

			}

		});

	};

	this.insertConfig = function () {

		// create column config
		var $colSetting = createColumnsConfig(
				this.localSetting( 'columns', tableColumns ),
				this.i18.config.label_columns,
				$.extend({timerfn:	'Buttons'}, this.i18.columns),
				'#content .timers',
				function(cfg) {
					self.setLocalSetting('columns', cfg );
				}
		);

		// logo/text
		var $channelSetting = createChannelConfig(
				this.localSetting('channel_display', 3),
				this.i18.config.channel_display.caption,
				this.i18.config.channel_display,
				null,
				function(cfg) {
					self.setLocalSetting('channel_display', cfg );
					$('#content .timers').attr('channel_mode', cfg);
				}
		);

		var $filterSetting = $(
			'<div class="setting">' +
				'<span class="label">' + this.i18.config.filter.caption + '</span>' +
				'<input type="radio" name="timerfilter" id="timerfilter0" value="0"><label for="timerfilter0">' + this.i18.config.filter.all + '</label> ' +
				'<input type="radio" name="timerfilter" id="timerfilter1" value="1"><label for="timerfilter1">' + this.i18.config.filter.rec + '</label> ' +
				'<input type="radio" name="timerfilter" id="timerfilter2" value="2"><label for="timerfilter2">' + this.i18.config.filter.play + '</label> ' +
			'</div>'
		);
		$filterSetting.find('INPUT[name="timerfilter"]').click(function() {

			var timersFilter = $('DIV#config INPUT[name="timerfilter"]:checked').val();
			self.setSetting( 'timersfilter',  timersFilter);

			$('.timers').attr('filter', timersFilter);

		}).filter('[value="' +  this.setting('timersfilter', 0) + '"]' ).prop('checked', true);

		$('DIV#config')
		.append($colSetting)
		.append($channelSetting)
		.append($filterSetting);

	};

}
